#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#
from ..outils import Geometrie as geo
import random,math
from ..outils.Affichage import decimaux
from ..outils.Geometrie import cotation, cotation_h

#trigo en degré
tan =lambda z:math.tan(math.radians(z))
cos =lambda z:math.cos(math.radians(z))
sin =lambda z:math.sin(math.radians(z))
def shuffle_nom(polygone):
    """renvoie un nom aléatoire du polygone"""
    n=len(polygone)
    polygone=polygone+polygone+polygone
    debut=n+random.randrange(n)
    sens=[-1,1][random.randrange(2)]
    nom=""
    for i in range(debut,debut+n*sens,sens):
        nom+=polygone[i]
    return nom

def exo_triangle(test=False):
    questions=[quest_equilateral,
           quest_LAL,
           quest_ALA,
           quest_AAL,
           quest_isocele_angbase,
           quest_isocele_angprincipal,
           quest_rectangle_hypo_cote,
           quest_rectangle_hypo_angle,
           ]
    exo=["\\exercice",
         "\\begin{enumerate}"]
    cor=["\\exercice*",
         "\\begin{enumerate}"]
    cor.append("\\definecolor{enonce}{rgb}{0.11,0.56,0.98}")
    cor.append("\\definecolor{calcul}{rgb}{0.13,0.54,0.13}")
    cor.append(u"\\psset{MarkAngleRadius=0.6,PointSymbol=none}")
    if test:#toutes les constructions en test
        for quest in questions:
            quest(exo,cor)
    else:#Construction choisies au hasard
        random.shuffle(questions)
        for i in range(4):
            questions[i](exo,cor)

    exo.append("\\end{enumerate}")
    cor.append("\\end{enumerate}")
    return exo,cor

def quest_equilateral(exo,cor):
    A,B,C=geo.choix_points(3)
    nom=shuffle_nom([A,B,C])
    c=0.1*random.randint(40,70)#longueur AB
    angBAC=angABC=60
    exo.append(u"\\item Trace un triangle $%s$ équilatéral de côté $\\unit[%s]{cm}$.\\par"%(nom,decimaux(c)))
    cor.append(u"\\item Trace un triangle $%s$ équilatéral de côté $\\unit[%s]{cm}$.\\par"%(nom,decimaux(c)))
    x_C=(c*tan(angABC))/(tan(angABC)+tan(angBAC))
    y_C=x_C*tan(angBAC)
    cor.append(u"\\begin{pspicture}(-1,-1)(%s,%s)"%(c+1,y_C+1))
    cor.append(u"\\pstTriangle(0,0){%s}(%s,0){%s}(%s,%s){%s}"%(A,c,B,x_C,y_C,C))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}"%(C,A))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}"%(B,C))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}"%(A,B))
    cor.append(u"\\pstRotation[RotAngle=7,PointSymbol=none,PointName=none]{%s}{%s}[C_1]"%(A,C))
    cor.append(u"\\pstRotation[RotAngle=-7,PointSymbol=none,PointName=none]{%s}{%s}[C_2]"%(A,C))
    cor.append(u"\\pstArcOAB[linecolor=calcul]{%s}{C_2}{C_1}"%(A))
    cor.append(u"\\pstRotation[RotAngle=7,PointSymbol=none,PointName=none]{%s}{%s}[C_3]"%(B,C))
    cor.append(u"\\pstRotation[RotAngle=-7,PointSymbol=none,PointName=none]{%s}{%s}[C_4]"%(B,C))
    cor.append(u"\\pstArcOAB[linecolor=calcul]{%s}{C_4}{C_3}"%(B))
    cor.append(cotation((0,0),(c,0),decimaux(c),couleur="enonce"))
    cor.append(u"\\end{pspicture}")

def quest_LAL(exo,cor):
    """on donne un angle et les longueurs de ses deux côtés"""
    """            angBAC et      AB=c et AC=b"""
    A,B,C=geo.choix_points(3)
    nom=shuffle_nom([A,B,C])
    c=0.1*random.randint(40,70)#longueur AB
    b=0.1*random.randint(20,100)#longueur BC
    angBAC=3*random.randint(7,50)#BAC mesure entre 21° et 150°
    
    exo.append(u"\\item Trace un triangle $%s$ tel que $%s%s=\\unit[%s]{cm}$, $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$"
               %(nom,A,B,decimaux(c),A,C,decimaux(b),B,A,C,angBAC))
    cor.append(u"\\item Trace un triangle $%s$ tel que $%s%s=\\unit[%s]{cm}$, $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par"
               %(nom,A,B,decimaux(c),A,C,decimaux(b),B,A,C,angBAC))
    cor.append(u"\\begin{pspicture}(%s,%s)(%s,%s)"%(min(0,b*cos(angBAC))-0.4,-1,max(b,b*cos(angBAC))+0.4,b*sin(angBAC)+1))
    cor.append(u"\\pstTriangle(0,0){%s}(%s;%s){%s}(%s,0){%s}"%(A,b,angBAC,C,c,B))
    cor.append(u"\\color{enonce}\\pstMarkAngle[linecolor=enonce]{%s}{%s}{%s}{%s\\degres}"%(B,A,C,angBAC))
    cor.append(u"\\pstLineAB[nodesepB=-1]{%s}{%s}"%(A,C))
    cor.append(u"\\pstRotation[RotAngle=7,PointSymbol=none,PointName=none]{%s}{%s}[C_1]"%(A,C))
    cor.append(u"\\pstRotation[RotAngle=-7,PointSymbol=none,PointName=none]{%s}{%s}[C_2]"%(A,C))
    cor.append(u"\\pstArcOAB[linecolor=calcul]{%s}{C_2}{C_1}"%(A))
    cor.append(cotation((0,0),(c,0),decimaux(c),couleur="enonce"))
    if angBAC<111:
        x_C,y_C=b*cos(angBAC),b*sin(angBAC)
        cor.append(cotation_h((0,0),(x_C,y_C),decimaux(b),couleur="enonce"))
    else:
        x_C,y_C=b*cos(angBAC),b*sin(angBAC)
        cor.append(cotation((x_C,y_C),(0,0),decimaux(b),couleur="enonce"))
    cor.append(u"\\end{pspicture}")

def quest_ALA(exo,cor):
    """on donne deux angles et la longueur du côté commun"""
    """ angBAC et angABC et AB=c"""
    A,B,C=geo.choix_points(3)
    nom=shuffle_nom([A,B,C])
    c=0.1*random.randint(40,70)#longueur AB
    angBAC=5*random.randint(4,12)
    angABC=5*random.randint(4,12)
    exo.append(u"\\item Trace un triangle $%s$ tel que $%s%s=\\unit[%s]{cm}$,  $\\widehat{%s%s%s}=%s\\degres$ et $\\widehat{%s%s%s}=%s\\degres$"
               %(nom,A,B,decimaux(c),B,A,C,angBAC,A,B,C,angABC))
    cor.append(u"\\item Trace un triangle $%s$ tel que $%s%s=\\unit[%s]{cm}$,  $\\widehat{%s%s%s}=%s\\degres$ et $\\widehat{%s%s%s}=%s\\degres$\\par"
               %(nom,A,B,decimaux(c),B,A,C,angBAC,A,B,C,angABC))
    x_C=(c*tan(angABC))/(tan(angABC)+tan(angBAC))
    y_C=x_C*tan(angBAC)
    cor.append(u"\\begin{pspicture}(-0.4,-1)(%s,%s)"%(c+1,y_C+1))
    cor.append(u"\\pstTriangle(0,0){%s}(%s,0){%s}(%s,%s){%s}"%(A,c,B,x_C,y_C,C))
    cor.append(u"\\pstLineAB[nodesepB=-1]{%s}{%s}\\pstLineAB[nodesepB=-1]{%s}{%s}"%(A,C,B,C))
    cor.append(cotation((0,0),(c,0),decimaux(c),couleur="enonce"))
    cor.append(u"\\color{enonce}\\pstMarkAngle[linecolor=enonce]{%s}{%s}{%s}{%s\\degres}"%(B,A,C,angBAC))
    cor.append(u"\\color{enonce}\\pstMarkAngle[linecolor=enonce]{%s}{%s}{%s}{%s\\degres}"%(C,B,A,angABC))
    cor.append(u"\\end{pspicture}")

def quest_AAL(exo,cor):
    """on donne deux angles et la longueur d'un côté non commun aux deux angles"""
    """ angBAC et angACB et AB=c"""
    A,B,C=geo.choix_points(3)
    nom=shuffle_nom([A,B,C])
    angBAC=3*random.randint(10,24)#entre 30° et 72°
    angACB=3*random.randint(10,24)#entre 30° et 72°
    angABC=180-angBAC-angACB
    ABmax=int(35*(tan(angABC)+tan(angBAC))/(tan(angBAC)*tan(angABC)))#donne une hauteur inférieur à 35*0.2=7 cm
    c=0.2*random.randint(20,max(20,ABmax))#longueur AB
    
    exo.append(u"\\item Trace un triangle $%s$ tel que $%s%s=\\unit[%s]{cm}$,  $\\widehat{%s%s%s}=%s\\degres$ et $\\widehat{%s%s%s}=%s\\degres$"
               %(nom,A,B,decimaux(c),B,A,C,angBAC,A,C,B,angACB))
    cor.append(u"\\item Trace un triangle $%s$ tel que $%s%s=\\unit[%s]{cm}$,  $\\widehat{%s%s%s}=%s\\degres$ et $\\widehat{%s%s%s}=%s\\degres$\\par"
               %(nom,A,B,decimaux(c),B,A,C,angBAC,A,C,B,angACB))
    cor.append(u"On doit d'abord calculer la mesure de $\\widehat{%s%s%s}$.\\\\"
               %(A,B,C))
    cor.append(u"Or la somme des trois angles d'un triangle est égale à 180\\degres donc $\\widehat{%s%s%s}=180\\degres-%s\\degres-%s\\degres=%s\\degres$.\\par"
               %(A,B,C,angBAC,angACB,angABC))
    x_C=(c*tan(angABC))/(tan(angABC)+tan(angBAC))
    y_C=x_C*tan(angBAC)
    cor.append(u"\\begin{pspicture}(-1,-1)(%s,%s)"%(c+1,y_C+1))
    cor.append(u"\\pstTriangle(0,0){%s}(%s,0){%s}(%s,%s){%s}"%(A,c,B,x_C,y_C,C))
    cor.append(u"\\pstLineAB[nodesepB=-1]{%s}{%s}\\pstLineAB[nodesepB=-1]{%s}{%s}"%(A,C,B,C))
    cor.append(u"\\pstMarkAngle[linecolor=calcul]{%s}{%s}{%s}{\\color{calcul}%s\\degres}"%(C,B,A,angABC))
    cor.append(u"\\color{enonce}\\pstMarkAngle[linecolor=enonce]{%s}{%s}{%s}{%s\\degres}"%(A,C,B,angACB))
    cor.append(cotation((0,0),(c,0),decimaux(c),couleur="enonce"))
    cor.append(u"\\pstMarkAngle[linecolor=enonce]{%s}{%s}{%s}{%s\\degres}"%(B,A,C,angBAC))
    cor.append(u"\\end{pspicture}")

def quest_isocele_angbase(exo,cor):
    """on donne ABC isocele en C, AB=c et angBAC"""
    A,B,C=geo.choix_points(3)
    nom=shuffle_nom([A,B,C])
    angABC=angBAC=random.randint(20,70)
    maxc = max(20,int(35 / tan(angBAC)))#longueur c maximale pour que la hauteur soit 7 cm =0.2*35
    minc = min(20,int(35 / tan(angBAC)))
    c=0.2*random.randint(minc,min(maxc,50))#longueur AB
    #Calcul pour tracer
    x_C=c/2
    y_C=x_C*tan(angBAC)

    exo.append(u"\\item Trace un triangle $%s$ isocèle en $%s$ tel que $%s%s=\\unit[%s]{cm}$,  $\\widehat{%s%s%s}=%s\\degres$."
               %(nom,C,A,B,decimaux(c),B,A,C,angBAC))
    cor.append(u"\\item Trace un triangle $%s$ isocèle en $%s$ tel que $%s%s=\\unit[%s]{cm}$,  $\\widehat{%s%s%s}=%s\\degres$. \\par"
               %(nom,C,A,B,decimaux(c),B,A,C,angBAC))
    cor.append(u"Comme $%s%s%s$ est un triangle isocèle en $%s$, je sais que les angles adjacents à la base sont de même mesure \
donc $\\widehat{%s%s%s}=\\widehat{%s%s%s}=%s\\degres$.\\par"%(A,B,C,C,A,B,C,B,A,C,angBAC))
    cor.append(u"\\begin{pspicture}(-1,-1)(%s,%s)"%(c+1,y_C+1))
    cor.append(u"\\pstTriangle(0,0){%s}(%s,0){%s}(%s,%s){%s}"%(A,c,B,x_C,y_C,C))
    cor.append(u"\\pstLineAB[nodesepB=-1]{%s}{%s}"%(A,C))
    cor.append("\\pstLineAB[linestyle=none]{%s}{%s} \\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}"%(A,C))
    cor.append(u"\\pstLineAB[nodesepB=-1]{%s}{%s}"%(B,C))
    cor.append(u"\\pstLineAB[linestyle=none]{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}"%(B,C))
    cor.append(cotation((0,0),(c,0),decimaux(c),couleur="enonce"))
    cor.append(u"\\color{enonce}\\pstMarkAngle[linecolor=enonce]{%s}{%s}{%s}{%s\\degres}"%(B,A,C,angBAC))
    cor.append(u"\\color{calcul}\\pstMarkAngle[linecolor=calcul]{%s}{%s}{%s}{%s\\degres}"%(C,B,A,angABC))
    cor.append(u"\\end{pspicture}")

def quest_isocele_angprincipal(exo,cor):
    """on donne ABC isocele en C, AB=c et angACB"""
    A,B,C=geo.choix_points(3)
    nom=shuffle_nom([A,B,C])
    angACB=2*random.randint(20,60)#ACB est pair entre 40° et 120°
    angBAC=angABC=(180-angACB)/2
    maxc = max(20,int(35 / tan(angBAC)))#longueur c maximale pour que la hauteur soit 7 cm =0.2*35
    c=0.2*random.randint(20,min(maxc,35))#longueur AB
    #Calcul pour tracer
    x_C=c/2
    y_C=x_C*tan(angBAC)
    
    exo.append(u"\\item Trace un triangle $%s$ isocèle en $%s$ tel que $%s%s=\\unit[%s]{cm}$,  $\\widehat{%s%s%s}=%s\\degres$."
               %(nom,C,A,B,decimaux(c),A,C,B,angACB))
    cor.append(u"\\item Trace un triangle $%s$ isocèle en $%s$ tel que $%s%s=\\unit[%s]{cm}$,  $\\widehat{%s%s%s}=%s\\degres$.\\par"
               %(nom,C,A,B,decimaux(c),A,C,B,angACB))
    cor.append(u"Comme $%s%s%s$ est un triangle isocèle en $%s$, je sais que les angles adjacents à la base sont de même mesure \
donc $\\widehat{%s%s%s}=\\widehat{%s%s%s}$.\\par"%(A,B,C,C,A,B,C,B,A,C))
    cor.append(u"De plus, je sais que la somme des mesures des trois angles d'un triangle est égale à 180\\degres \\\\ \
donc $\\widehat{%s%s%s}=\\widehat{%s%s%s}=(180\\degres-%s\\degres)\\div 2=%s\\degres$. \\par"
               %(B,A,C,A,B,C,angACB,angBAC))
    cor.append(u"\\begin{pspicture}(-1,-1)(%s,%s)"%(c+1,y_C+1))
    cor.append(u"\\pstTriangle(0,0){%s}(%s,0){%s}(%s,%s){%s}"%(A,c,B,x_C,y_C,C))
    cor.append(u"\\pstLineAB[nodesepB=-1]{%s}{%s}"%(A,C))
    cor.append("\\pstLineAB[linestyle=none]{%s}{%s} \\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}"%(A,C))
    cor.append(u"\\pstLineAB[nodesepB=-1]{%s}{%s}"%(B,C))
    cor.append("\\pstLineAB[linestyle=none]{%s}{%s} \\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}"%(B,C))
    cor.append(cotation((0,0),(c,0),decimaux(c),couleur="enonce"))
    cor.append(u"\\color{enonce}\\pstMarkAngle[linecolor=enonce,Mark=MarkHash]{%s}{%s}{%s}{%s\\degres}"%(A,C,B,angACB))
    cor.append(u"\\color{calcul}\\pstMarkAngle[linecolor=calcul]{%s}{%s}{%s}{%s\\degres}"%(B,A,C,angBAC))
    cor.append(u"\\pstMarkAngle[linecolor=calcul]{%s}{%s}{%s}{%s\\degres}"%(C,B,A,angABC))
    cor.append(u"\\end{pspicture}")
    
def quest_rectangle_hypo_angle(exo,cor):
    """on donne un triangle ABC rectangle en C et l'hypotenuse AB et l'angle BAC"""
    A,B,C=geo.choix_points(3)
    nom=shuffle_nom([A,B,C])
    c=0.2*random.randint(20,35)#longueur AB
    angBAC=3*random.randint(7,23)#un angle mesurant entre 21° et 69°
    angABC=90-angBAC
    #angACB=90
    #Calcul pour tracer
    x_C=(c*tan(angABC))/(tan(angABC)+tan(angBAC))
    y_C=x_C*tan(angBAC)
    
    exo.append(u"\\item Trace un triangle $%s$ rectangle en $%s$ tel que $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par"
               %(nom,C,A,B,decimaux(c),B,A,C,angBAC))
    cor.append(u"\\item Trace un triangle $%s$ rectangle en $%s$ tel que $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par"
               %(nom,C,A,B,decimaux(c),B,A,C,angBAC))
##    cor.append("\\begin{multicols}{2}")
    cor.append(u"Je sais que dans un triangle rectangle, les deux angles aigus sont complémentaires \\\\ \
donc $\widehat{%s%s%s}=90\\degres-%s\\degres=%s\\degres$.\\par"%(B,A,C,angBAC,angABC))
    
    cor.append("\\figureadroite{")
    cor.append(u"\\begin{pspicture}(-0.4,-1)(%s,%s)"%(c+0.4,y_C+1))
    cor.append(u"\\pstTriangle(0,0){%s}(%s,0){%s}(%s,%s){%s}"%(A,c,B,x_C,y_C,C))
    cor.append(u"\\color{enonce}\\pstRightAngle[linecolor=enonce]{%s}{%s}{%s}"%(A,C,B))
    cor.append(u"\\pstLineAB[nodesepB=-1]{%s}{%s}\\pstLineAB[nodesepB=-1]{%s}{%s}"%(A,C,B,C))
    cor.append(cotation((0,0),(c,0),decimaux(c),couleur="enonce"))
    cor.append(u"\\color{enonce}\\pstMarkAngle[linecolor=enonce]{%s}{%s}{%s}{%s\\degres}"%(B,A,C,angBAC))
    cor.append(u"\\color{calcul}\\pstMarkAngle[linecolor=calcul]{%s}{%s}{%s}{%s\\degres}"%(C,B,A,angABC))
    cor.append(u"\\end{pspicture}}{")
    cor.append("\\begin{enumerate}")
    cor.append(u"\\item Je trace le segment $[%s%s]$ mesurant $\\unit[%s]{cm}$ ;"%(A,B,decimaux(c)))
    cor.append(u"\\item puis la demi-droite $[%s%s)$ en traçant l'angle~$\widehat{%s%s%s}$ ;"%(A,C,B,A,C))
    cor.append(u"\\item puis la demi-droite $[%s%s)$ en traçant l'angle~$\widehat{%s%s%s}$ ;"%(B,C,A,B,C))
##    cor.append(u"\\item enfin je vérifie les trois {\\color{enonce}conditions de l'énoncé}.")
    cor.append("\\end{enumerate}\n")
##    cor.append("\\end{multicols}")
    cor.append("}")
    
def quest_rectangle_hypo_cote(exo,cor):
    """on donne un triangle ABC rectangle en B et l'hypotenuse AC et le coté AB"""
    A,B,C=geo.choix_points(3)
    nom=shuffle_nom([A,B,C])
    c=0.2*random.randint(20,35)#longueur AB
    b=0.1*random.randint(int(10*(c))+1,100)#longueur AC
    angABC=90
    #calcul pour tracer
    angBAC=math.degrees(math.acos(float(c)/float(b)))
    x_C=(c*tan(angABC))/(tan(angABC)+tan(angBAC))
    y_C=x_C*tan(angBAC)
    
    exo.append(u"\\item Trace un triangle $%s$ rectangle en $%s$ tel que $%s%s=\\unit[%s]{cm}$,  $%s%s=\\unit[%s]{cm}$.\\par"
               %(nom,B,A,B,decimaux(c),A,C,decimaux(b)))
    cor.append(u"\\item Trace un triangle $%s$ rectangle en $%s$ tel que $%s%s=\\unit[%s]{cm}$, $%s%s=\\unit[%s]{cm}$.\\par"
               %(nom,B,A,B,decimaux(c),A,C,decimaux(b)))
##    cor.append("\\begin{multicols}{2}")
    cor.append("\\figureadroite{")
    cor.append(u"\\begin{pspicture}(-0.4,-1)(%s,%s)"%(c+0.4,y_C+1))
    cor.append(u"\\pstTriangle(0,0){%s}(%s,0){%s}(%s,%s){%s}"%(A,c,B,x_C,y_C,C))
    cor.append(u"\\color{enonce}\\pstRightAngle[linecolor=enonce]{%s}{%s}{%s}"%(C,B,A))
    cor.append(u"\\pstRotation[RotAngle=7,PointSymbol=none,PointName=none]{%s}{%s}[C_1]"%(A,C))
    cor.append(u"\\pstRotation[RotAngle=-7,PointSymbol=none,PointName=none]{%s}{%s}[C_2]"%(A,C))
    cor.append(u"\\pstArcOAB[linecolor=calcul]{%s}{C_2}{C_1}"%(A))
    cor.append(u"\\pstLineAB{%s}{%s}\\pstLineAB[nodesepB=-1]{%s}{%s}"%(A,B,B,C))
    cor.append(cotation((0,0),(c,0),decimaux(c),couleur="enonce"))
    cor.append(cotation_h((0,0),(x_C,y_C),decimaux(b),couleur="enonce"))
    cor.append(u"\\end{pspicture}}{")
    cor.append(u"\\begin{enumerate}")
    cor.append(u"\\item Je trace le segment $[%s%s]$ mesurant $\\unit[%s]{cm}$ ;"%(A,B,decimaux(c)))
    cor.append(u"\\item puis je trace l'angle droit $\\widehat{%s%s%s}$ ;"%(A,B,C))
    cor.append(u"\\item enfin, je reporte au compas la longueur \\mbox{$%s%s=\\unit[%s]{cm}$} à partir de $%s$."%(A,C,decimaux(b),A))
    cor.append("\\end{enumerate}}")
##    cor.append("\\end{multicols}")


#################################################################################################
#   Les construction sont faites avec [AB] horizontale (ou avec la diagonale [AC] horizontale)
#       D-------C
#      /       /
#     /       /
#    A-------B
#   A est toujours l'origine : A(0,0)
#
#   AB est la longueur AB : type(float) en cm
#   angBAC est la mesure de l'angle BAC : type(int) en degrés
#   x_C,y_C sont les coordonnées cartésiennes du point C
#  les coordonnées polaires sont notées (5 ; 60)
#
###################################################################################################

def exo_quadrilatere(test=False):
    
    exo=["\\exercice",
         "\\begin{enumerate}"]
    cor=["\\exercice*",
         "\\begin{enumerate}"]
    cor.append("\\definecolor{enonce}{rgb}{0.11,0.56,0.98}")
    #couleur pour reporter les informations de la consigne
    cor.append("\\definecolor{calcul}{rgb}{0.13,0.54,0.13}")
    #couleur pour reporter des informations déduites par raisonnement ou calcul
    cor.append(u"\\psset{MarkAngleRadius=0.7,PointSymbol=none,dotscale=2}")
    #MarkAngleRadius = rayon de l'arc marquant les angles
    #PointSymbol=none les points ne sont pas tracés
    #dotscale=2, grossit la  croix si PointSymbol=x

    #liste des questions possibles
    questions=[quest_rectangle_diag,
           quest_rectangle_angle,
           quest_rectangle_angle_diag,
           quest_rectangle_diag_angle,
           quest_parallelogramme_CCA,
           quest_parallelogramme_CDA,
           quest_parallelogramme_DDA,
           quest_losange_DD,
           quest_losange_CD,
           quest_losange_CDbis,
           quest_losange_CC,
           carre_diag]

    if test:#toutes les constructions en test
        for quest in questions:
            quest(exo,cor)
    else:
        #on choisit un parallélogramme, un losange et un rectangle
        questions[random.randrange(4)](exo,cor)
        questions[random.randrange(4,7)](exo,cor)
        questions[random.randrange(7,11)](exo,cor)
        #questions[11], le carré n'est jamais proposé
    exo.append("\\end{enumerate}")
    cor.append("\\end{enumerate}")
    return exo,cor

################################################################

####### RECTANGLES #############

def quest_rectangle_diag(exo,cor):
    """on donne un rectangle ABCD avec un côté AB et une diagonale AC"""
    A,B,C,D=geo.choix_points(4)
    nom=shuffle_nom([A,B,C,D])
    L=random.randint(40,60)#AB mesure entre 4cm et 7cm, tracé horizontalement
    Diag=0.1*random.randint(L+10,70)#+1.1 pour éviter les problèmes d'arrondi
    L=0.1*L
    #Calcul pour tracer
    angBAC=math.degrees(math.acos(float(L)/float(Diag)))
    x_C=L
    y_C=x_C*tan(angBAC)
    
    exo.append(u"\\item Trace un rectangle $%s$ tel que $%s%s=\\unit[%s]{cm}$ et $%s%s=\\unit[%s]{cm}$.\\par"
               %(nom,A,B,decimaux(L),A,C,decimaux(Diag)))
    cor.append(u"\\item Trace un rectangle $%s$ tel que $%s%s=\\unit[%s]{cm}$ et $%s%s=\\unit[%s]{cm}$.\\par"
               %(nom,A,B,decimaux(L),A,C,decimaux(Diag)))
    #figure
    cor.append("\\figureadroite{")
    cor.append(u"\\begin{pspicture}(-0.4,-1)(%s,%s)"%(L+0.4,y_C+1))
    cor.append(u"\\pstTriangle(0,0){%s}(%s,0){%s}(%s,%s){%s}"%(A,L,B,x_C,y_C,C))
    cor.append(u"\\pstGeonode[PosAngle=135](0,%s){%s}"%(y_C,D))
    #codage
    cor.append(u"\\color{enonce}\\pstRightAngle[linecolor=enonce]{%s}{%s}{%s}"%(D,C,B))
    cor.append(u"\\color{enonce}\\pstRightAngle[linecolor=enonce]{%s}{%s}{%s}"%(B,A,D))
    cor.append(u"\\color{enonce}\\pstRightAngle[linecolor=enonce]{%s}{%s}{%s}"%(C,B,A))
    cor.append(u"\\pstRotation[RotAngle=7,PointSymbol=none,PointName=none]{%s}{%s}[C_1]"%(A,C))
    cor.append(u"\\pstRotation[RotAngle=-7,PointSymbol=none,PointName=none]{%s}{%s}[C_2]"%(A,C))
    cor.append(u"\\pstArcOAB[linecolor=calcul]{%s}{C_2}{C_1}"%(A))
    cor.append(u"\\pstLineAB[nodesepB=-1]{%s}{%s}\\pstLineAB[nodesepB=-1]{%s}{%s}\\pstLineAB{%s}{%s}\\pstLineAB[nodesepB=-1]{%s}{%s}"%(A,D,C,D,A,B,B,C))
    cor.append(cotation((0,0),(L,0),decimaux(L),couleur="enonce"))
    cor.append(cotation_h((0,0),(x_C,y_C),decimaux(Diag),couleur="enonce"))
    cor.append(u"\\end{pspicture}}{")
    #Programme de construction
    cor.append(u"\\begin{enumerate}")
    cor.append(u"\\item Je trace le segment $[%s%s]$ mesurant $\\unit[%s]{cm}$ ;"%(A,B,decimaux(L)))
    cor.append(u"\\item puis je trace l'angle droit $\\widehat{%s%s%s}$ ;"%(A,B,C))
    cor.append(u"\\item je reporte au compas la longueur $%s%s=\\unit[%s]{cm}$ à partir de $%s$ ;"%(A,C,decimaux(Diag),A))
    cor.append(u"\\item je trace enfin les angles droits en $%s$ et en $%s$ pour placer le point $%s$."%(A,C,D))
    cor.append("\\end{enumerate}}")
    

def quest_rectangle_angle(exo,cor):
    """On donne un rectangle ABCD avec le côté AB et l'angle BAC"""
    A,B,C,D=geo.choix_points(4)
    nom=shuffle_nom([A,B,C,D])
    L=random.randint(40,60)#AB mesure entre 4cm et 7cm, tracé horizontalement
    angBAC=random.randint(25,65)
    L=0.1*L
    #Calcul pour tracer
    x_C=L
    y_C=x_C*tan(angBAC)
    
    exo.append(u"\\item Trace un rectangle $%s$ tel que $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par"
               %(nom,A,B,decimaux(L),B,A,C,angBAC))
    cor.append(u"\\item Trace un rectangle $%s$ tel que $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par"
               %(nom,A,B,decimaux(L),B,A,C,angBAC))
    
    cor.append("\\figureadroite{")
    cor.append(u"\\begin{pspicture}(-0.4,-1)(%s,%s)"%(L+0.4,y_C+1))
    cor.append(u"\\pstTriangle(0,0){%s}(%s,0){%s}(%s,%s){%s}"%(A,L,B,x_C,y_C,C))
    cor.append(u"\\pstGeonode[PosAngle=135](0,%s){%s}"%(y_C,D))
    cor.append(u"\\color{enonce}\\pstRightAngle[linecolor=enonce]{%s}{%s}{%s}"%(D,C,B))
    cor.append(u"\\color{enonce}\\pstRightAngle[linecolor=enonce]{%s}{%s}{%s}"%(B,A,D))
    cor.append(u"\\color{enonce}\\pstRightAngle[linecolor=enonce]{%s}{%s}{%s}"%(C,B,A))
    cor.append(u"\\pstLineAB[nodesepB=-1]{%s}{%s}\\pstLineAB[nodesepB=-1]{%s}{%s}\\pstLineAB{%s}{%s}\\pstLineAB[nodesepB=-1]{%s}{%s}\\pstLineAB[nodesepB=-1]{%s}{%s}"
               %(A,D,C,D,A,B,B,C,A,C))
    cor.append(cotation((0,0),(L,0),decimaux(L),couleur="enonce"))
    cor.append(u"\\color{enonce}\\pstMarkAngle[linecolor=enonce]{%s}{%s}{%s}{%s\\degres}"%(B,A,C,angBAC))
    cor.append(u"\\end{pspicture}}{")
    #Programme de construction
    cor.append(u"\\begin{enumerate}")
    cor.append(u"\\item Je trace le segment $[%s%s]$ mesurant $\\unit[%s]{cm}$ ;"%(A,B,decimaux(L)))
    cor.append(u"\\item puis je trace l'angle droit $\\widehat{%s%s%s}$ ;"%(A,B,C))
    cor.append(u"\\item la demi-droite $[%s%s)$ en mesurant $\\widehat{%s%s%s}=%s\\degres$."%(A,C,B,A,C,angBAC))
    cor.append(u"\\item je trace enfin les angles droit en $%s$ et en $%s$ pour placer le point $%s$."%(A,C,D))
    cor.append("\\end{enumerate}}")

def quest_rectangle_angle_diag(exo,cor):
    """On donne un rectangle ABCD de centre E, la diagonale AC et l'angle AEB"""
    A,B,C,D,E=geo.choix_points(5)
    nom=shuffle_nom([A,B,C,D])
    angAEB=2*random.randint(20,70)
    Diag=0.2*random.randint(25,45)
    #calcul pour tracer
    angBAC=(180-angAEB)/2
    L=Diag*cos(angBAC)
    x_C=L
    y_C=x_C*tan(angBAC)
    
    exo.append(u"\\item Trace un rectangle $%s$ de centre $%s$ tel que $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par"
               %(nom,E,A,C,decimaux(Diag),A,E,B,angAEB))
    cor.append(u"\\item Trace un rectangle $%s$ de centre $%s$ tel que $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par"
               %(nom,E,A,C,decimaux(Diag),A,E,B,angAEB))
##    cor.append("\\begin{multicols}{2}")
    #Programme de construction
    cor.append("\\figureadroite{")
##    cor.append("\columnbreak")
    cor.append(u"\\begin{pspicture}(-0.4,-1)(%s,%s)"%(L+0.4,y_C+1))
    cor.append(u"\\pstTriangle(0,0){%s}(%s,0){%s}(%s,%s){%s}"%(A,L,B,x_C,y_C,C))
    cor.append(u"\\pstGeonode[PosAngle={135,0}](0,%s){%s}(%s,%s){%s}"%(y_C,D,x_C/2.0,y_C/2.0,E))
    cor.append(u"\\pstLineAB{%s}{%s}"%(C,D))
    cor.append(u"\\pstLineAB{%s}{%s}"%(A,D))
    cor.append(u"\\pstLineAB[nodesep=-1]{%s}{%s}"%(B,D))
    #codage
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkHashh}"%(E,A))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkHashh}"%(E,B))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkHashh}"%(E,C))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkHashh}"%(E,D))
    cor.append(u"\\psarc[linecolor=calcul](%s,%s){%s}{%s}{%s} "%(x_C/2.0,y_C/2.0,Diag/2,angBAC-7,angBAC+7))
    cor.append(u"\\psarc[linecolor=calcul](%s,%s){%s}{%s}{%s} "%(x_C/2.0,y_C/2.0,Diag/2,180-angBAC-7,180-angBAC+7))
    cor.append(u"\\psarc[linecolor=calcul](%s,%s){%s}{%s}{%s} "%(x_C/2.0,y_C/2.0,Diag/2,-angBAC-7,-angBAC+7))
    cor.append(u"\\psarc[linecolor=calcul](%s,%s){%s}{%s}{%s} "%(x_C/2.0,y_C/2.0,Diag/2,180+angBAC-7,180+angBAC+7))
    cor.append("\\pcline[linestyle=none](0,0)(%s,%s)\\aput*{:U}{\\color{enonce}\\unit[%s]{cm}}"%(L,y_C,decimaux(Diag)))
    cor.append(u"\\color{enonce}\\pstMarkAngle[linecolor=enonce]{%s}{%s}{%s}{%s\\degres}"%(A,E,B,angAEB))
    cor.append(u"\\end{pspicture}}{")
    cor.append(u"\\begin{enumerate}")
    cor.append(u"\\item Je trace le segment $[%s%s]$ mesurant $\\unit[%s]{cm}$ ;"%(A,C,decimaux(Diag)))
    cor.append(u"\\item le centre du rectangle est le milieu des diagonales donc $%s$ est le milieu de $[%s%s]$ ;"%(E,A,C))
    cor.append(u"\\item je trace la diagonale $(%s%s)$ passant par $%s$ en mesurant \\mbox{$\\widehat{%s%s%s}=%s\\degres$} ;"
               %(B,D,E,A,E,B,angAEB))
    cor.append(u"\\item Comme les diagonales du rectangle sont de même longueur, je reporte les longueurs $%s%s=%s%s=\\unit[%s]{cm}$."
               %(E,D,E,B,decimaux(Diag/2)))
##    cor.append(u"\\item je trace enfin les angles droits en $%s$ et en $%s$ pour palcer le point $%s$."%(A,C,D))
    cor.append("\\end{enumerate}}")
##    cor.append("\\end{multicols}")

def quest_rectangle_diag_angle(exo,cor):
    """On donne un rectangle ABCD la diagonale AC et l'angle BAC"""
    A,B,C,D=geo.choix_points(4)
    nom=shuffle_nom([A,B,C,D])
    L=random.randint(40,60)#AB mesure entre 4cm et 7cm, tracé horizontalement
    angBAC=random.randint(25,65)
    Diag=0.1*random.randint(50,80)
    #Calcul
    L=Diag*cos(angBAC)
    x_C=L
    y_C=x_C*tan(angBAC)
    
    exo.append(u"\\item Trace un rectangle $%s$ tel que $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par"
               %(nom,A,C,decimaux(Diag),B,A,C,angBAC))
    cor.append(u"\\item Trace un rectangle $%s$ tel que $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par"
               %(nom,A,C,decimaux(Diag),B,A,C,angBAC))
    
##    cor.append("\\begin{multicols}{2}")
    cor.append("\\figureadroite{")
##    cor.append("\columnbreak")
    #figure
    cor.append(u"\\begin{pspicture}(-0.4,-1)(%s,%s)"%(L+0.4,y_C+1))
    cor.append(u"\\pstTriangle(0,0){%s}(%s,0){%s}(%s,%s){%s}"%(A,L,B,x_C,y_C,C))
    cor.append(u"\\pstGeonode[PosAngle=135](0,%s){%s}"%(y_C,D))
    #codage
    cor.append(u"\\color{enonce}\\pstRightAngle[linecolor=enonce]{%s}{%s}{%s}"%(D,C,B))
    cor.append(u"\\color{enonce}\\pstRightAngle[linecolor=enonce]{%s}{%s}{%s}"%(B,A,D))
    cor.append(u"\\color{enonce}\\pstRightAngle[linecolor=enonce]{%s}{%s}{%s}"%(C,B,A))
    cor.append(u"\\pstLineAB[nodesepB=-1]{%s}{%s}\\pstLineAB[nodesepB=-1]{%s}{%s}\\pstLineAB[nodesepB=-1]{%s}{%s}\\pstLineAB[nodesep=-1]{%s}{%s}"
               %(A,D,C,D,A,B,B,C))
    cor.append(cotation_h((0,0),(L,y_C),decimaux(Diag),couleur="enonce"))
    cor.append(u"\\color{enonce}\\pstMarkAngle[linecolor=enonce]{%s}{%s}{%s}{%s\\degres}"%(B,A,C,angBAC))
    cor.append(u"\\end{pspicture}}{")
    #Programme de construction
    cor.append(u"\\begin{enumerate}")
    cor.append(u"\\item Je trace le segment $[%s%s]$ mesurant $\\unit[%s]{cm}$ ;"%(A,C,decimaux(Diag)))
    cor.append(u"\\item la demi-droite $[%s%s)$ en mesurant \\mbox{$\\widehat{%s%s%s}=%s\\degres$} ;"%(A,B,B,A,C,angBAC))
    cor.append(u"\\item puis la perpendiculaire à $[%s%s)$ passant par~$%s$ ;"%(A,B,C))
    cor.append(u"\\item je trace enfin les angles droits en $%s$ et en $%s$ pour placer le point~$%s$."%(A,C,D))
    cor.append("\\end{enumerate}}")
##    cor.append("\\end{multicols}")

################## PARALLÉLOGRAMMES QUELCONQUES ###########################
    
def quest_parallelogramme_CCA(exo,cor):
    """On donne un parallélogramme avec la longueur de deux côtés et un angle."""
    #3 choix d'angle : CC ou un autre angle CC ou un angle Côté Diagonale
    A,B,C,D=geo.choix_points(4)
    nom=shuffle_nom([A,B,C,D])
    AB=0.1*random.randint(40,60)#AB mesure entre 4cm et 7cm, tracé horizontalement
    AD=0.1*random.randint(40,60)#AD mesure entre 4cm et 7cm, tracé horizontalement
    angBAD=random.randint(25,65)
    #Pour tracer
    x_C=AB+AD*cos(angBAD)
    y_C=AD*sin(angBAD)
    
    exo.append(u"\\item Trace un parallélogramme $%s$ tel que $%s%s=\\unit[%s]{cm}$, $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par"
               %(nom,A,B,decimaux(AB),D,A,decimaux(AD),B,A,D,angBAD))
    cor.append(u"\\item Trace un parallélogramme $%s$ tel que $%s%s=\\unit[%s]{cm}$, $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par"
               %(nom,A,B,decimaux(AB),D,A,decimaux(AD),B,A,D,angBAD))
    cor.append(u"\\begin{enumerate}")
    cor.append(u"\\item Je trace le segment $[%s%s]$ mesurant $\\unit[%s]{cm}$ ;"%(A,B,decimaux(AB)))
    cor.append(u"\\item je mesure l'angle  $\\widehat{%s%s%s}=%s\\degres$ puis je place le point~$%s$ ;"%(B,A,D,angBAD,D))
    cor.append(u"\\item enfin je reporte les longueurs $%s%s=%s%s$ et $%s%s=%s%s$ pour place le point~$%s$."
               %(D,C,A,B,B,C,A,D,C))
    cor.append("\\end{enumerate}\n")
    cor.append(u"\\begin{pspicture}(-0.4,-1)(%s,%s)"%(max(AB,x_C)+0.4,y_C+1))
    cor.append(u"\\pstGeonode[PosAngle={-135,-45,45,135}](0,0){%s}(%s,0){%s}(%s,%s){%s}(%s;%s){%s}"%(A,AB,B,x_C,y_C,C,AD,angBAD,D))
    cor.append(u"\\pstLineAB[nodesepB=-1]{%s}{%s}\\pstLineAB{%s}{%s}\\pstLineAB{%s}{%s}\\pstLineAB{%s}{%s}"
               %(A,D,C,D,A,B,B,C))
    cor.append(u"\\pstRotation[RotAngle=7,PointSymbol=none,PointName=none]{%s}{%s}[C_1]"%(D,C))
    cor.append(u"\\pstRotation[RotAngle=-7,PointSymbol=none,PointName=none]{%s}{%s}[C_2]"%(D,C))
    cor.append(u"\\pstArcOAB[linecolor=calcul]{%s}{C_2}{C_1}"%(D))
    cor.append(u"\\pstRotation[RotAngle=7,PointSymbol=none,PointName=none]{%s}{%s}[D_1]"%(A,D))
    cor.append(u"\\pstRotation[RotAngle=-7,PointSymbol=none,PointName=none]{%s}{%s}[D_2]"%(A,D))
    cor.append(u"\\pstArcOAB[linecolor=calcul]{%s}{D_2}{D_1}"%(A))
    cor.append(u"\\pstRotation[RotAngle=7,PointSymbol=none,PointName=none]{%s}{%s}[C_3]"%(B,C))
    cor.append(u"\\pstRotation[RotAngle=-7,PointSymbol=none,PointName=none]{%s}{%s}[C_4]"%(B,C))
    cor.append(u"\\pstArcOAB[linecolor=calcul]{%s}{C_4}{C_3}"%(B))
    cor.append(cotation_h((0,0),(x_C-AB,y_C),decimaux(AD),couleur="enonce"))
    cor.append(cotation((0,0),(AB,0),decimaux(AB),couleur="enonce"))
    cor.append(u"\\color{enonce}\\pstMarkAngle[linecolor=enonce]{%s}{%s}{%s}{%s\\degres}"%(B,A,D,angBAD))
    cor.append(u"\\pstLineAB[linestyle=none]{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkHashh}"%(A,D))
    cor.append(u"\\pstLineAB[linestyle=none]{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkHashh}"%(B,C))
    cor.append(u"\\pstLineAB[linestyle=none]{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkHash}"%(A,B))
    cor.append(u"\\pstLineAB[linestyle=none]{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkHash}"%(D,C))
    cor.append(u"\\end{pspicture}")

def quest_parallelogramme_CDA(exo,cor):
    """On donne un parallélogramme avec la longueur d'un côté et une diagonale et l'angle entre ces deux segments"""
    
    A,B,C,D=geo.choix_points(4)
    nom=shuffle_nom([A,B,C,D])
    AB=0.1*random.randint(40,60)#AB mesure entre 4cm et 7cm, tracé horizontalement
    AC=0.1*random.randint(40,70)#AC mesure entre 4cm et 7cm, tracé horizontalement
    angBAC=random.randint(25,65)
    #Calcul pour tracer
    x_C=round(AC*cos(angBAC),4)#round() évite une écriture scientifique si x_D=2.4e-15, non reconnue par PSTricks
    y_D=y_C=AC*sin(angBAC)
    x_D=round(x_C-AB,4)
    
    exo.append(u"\\item Trace un parallélogramme $%s$ tel que $%s%s=\\unit[%s]{cm}$, $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par"
               %(nom,A,B,decimaux(AB),C,A,decimaux(AC),B,A,C,angBAC))
    cor.append(u"\\item Trace un parallélogramme $%s$ tel que $%s%s=\\unit[%s]{cm}$, $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par"
               %(nom,A,B,decimaux(AB),C,A,decimaux(AC),B,A,C,angBAC))
    #Programme de construction
    cor.append(u"\\begin{enumerate}")
    cor.append(u"\\item Je trace le segment $[%s%s]$ mesurant $\\unit[%s]{cm}$ ;"%(A,B,decimaux(AB)))
    cor.append(u"\\item je trace la demi-droite $[%s%s)$ en mesurant $\\widehat{%s%s%s}=%s\\degres$ ;"%(A,C,B,A,C,angBAC))
    cor.append(u"\\item je place le point $%s$ en mesurant $%s%s=\\unit[%s]{cm}$ ;"%(C,A,C,decimaux(AC)))
    cor.append(u"\\item je construis le point $%s$ en reportant au compas $%s%s=%s%s$ et $%s%s=%s%s$."
               %(D,C,D,B,A,A,D,B,C))
    cor.append("\\end{enumerate}\n")
    #Figure
    cor.append(u"\\begin{pspicture}(%s,-1)(%s,%s)"%(min(0,x_C-AB)-0.4,max(AB,x_C)+0.4,y_C+1))
    cor.append(u"\\pstGeonode[PosAngle={-135,-45,45,135}](0,0){%s}(%s,0){%s}(%s,%s){%s}(%s,%s){%s}"%(A,AB,B,x_C,y_C,C,x_D,y_D,D))
    cor.append(u"\\pstLineAB{%s}{%s}\\pstLineAB{%s}{%s}\\pstLineAB{%s}{%s}\\pstLineAB{%s}{%s}\\pstLineAB[nodesepB=-1]{%s}{%s}"
               %(A,D,C,D,A,B,B,C,A,C))
    #Construction
    cor.append(u"\\pstRotation[RotAngle=7,PointSymbol=none,PointName=none]{%s}{%s}[C_1]"%(A,C))
    cor.append(u"\\pstRotation[RotAngle=-7,PointSymbol=none,PointName=none]{%s}{%s}[C_2]"%(A,C))
    cor.append(u"\\pstArcOAB[linecolor=calcul]{%s}{C_2}{C_1}"%(A))
    cor.append(u"\\pstRotation[RotAngle=7,PointSymbol=none,PointName=none]{%s}{%s}[D_1]"%(A,D))
    cor.append(u"\\pstRotation[RotAngle=-7,PointSymbol=none,PointName=none]{%s}{%s}[D_2]"%(A,D))
    cor.append(u"\\pstArcOAB[linecolor=calcul]{%s}{D_2}{D_1}"%(A))
    cor.append(u"\\pstRotation[RotAngle=7,PointSymbol=none,PointName=none]{%s}{%s}[D_3]"%(C,D))
    cor.append(u"\\pstRotation[RotAngle=-7,PointSymbol=none,PointName=none]{%s}{%s}[D_4]"%(C,D))
    cor.append(u"\\pstArcOAB[linecolor=calcul]{%s}{D_4}{D_3}"%(C))
    #Codage
    cor.append(cotation_h((0,0),(x_C,y_C),decimaux(AC),couleur="enonce"))
    cor.append(cotation((0,0),(AB,0),decimaux(AB),couleur="enonce"))
    cor.append(u"\\color{enonce}\\pstMarkAngle[linecolor=enonce]{%s}{%s}{%s}{%s\\degres}"%(B,A,C,angBAC))
    cor.append(u"\\pstLineAB[linestyle=none]{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkHashh}"%(A,D))
    cor.append(u"\\pstLineAB[linestyle=none]{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkHashh}"%(B,C))
    cor.append(u"\\pstLineAB[linestyle=none]{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkHash}"%(A,B))
    cor.append(u"\\pstLineAB[linestyle=none]{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkHash}"%(D,C))
    cor.append(u"\\end{pspicture}")

def quest_parallelogramme_DDA(exo,cor):
    """On donne un parallélogramme avec la longueur des deux diagonales et l'angle entre les diagonales"""
    # choix d'angle : on pourrait donner l'angle BAC
    #diagonale ou demi-diagonale
    A,B,C,D,E=geo.choix_points(5)
    nom=shuffle_nom([A,B,C,D])
    BD=0.2*random.randint(20,40)#AB mesure entre 4cm et 8cm, tracé horizontalement
    AC=0.2*random.randint(20,40)#AC mesure entre 4cm et 8cm, tracé horizontalement
    angAEB=random.randint(35,145)
    AE=AC/2
    BE=BD/2
    
    #calcul pour tracer
    AB=math.sqrt(AE**2+BE**2-2*AE*BE*cos(angAEB))
    angBAC=math.degrees(math.asin(BE*sin(angAEB)/AB))
    x_C=AC*cos(angBAC)
    y_D=y_C=round(AC*sin(angBAC),4)
    x_D=round(x_C-AB,4)
    
    exo.append(u"\\item Trace un parallélogramme $%s$ de centre $%s$ tel que $%s%s=\\unit[%s]{cm}$, $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par"
               %(nom,E,A,C,decimaux(AC),B,D,decimaux(BD),A,E,B,angAEB))
    cor.append(u"\\item Trace un parallélogramme $%s$ de centre $%s$ tel que $%s%s=\\unit[%s]{cm}$, $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par"
               %(nom,E,A,C,decimaux(AC),B,D,decimaux(BD),A,E,B,angAEB))
    cor.append(u"\\begin{enumerate}")
    cor.append(u"\\item Je trace le segment $[%s%s]$ mesurant $\\unit[%s]{cm}$ ;"%(A,C,decimaux(AC)))
    cor.append(u"\\item Dans un parallélogramme les diagonales se coupent en leur milieu donc $%s%s=%s%s=\\unit[%s]{cm}$ et $%s%s=%s%s=\\unit[%s]{cm}$ ;"
               %(A,E,C,E,decimaux(AC/2),B,E,E,D,decimaux(BD/2)))
    cor.append("\\end{enumerate}\n")
    cor.append(u"\\begin{pspicture}(%s,-1)(%s,%s)"%(min(0,x_D)-0.4,max(AB,x_C)+0.4,y_C+1))
    cor.append(u"\\pstGeonode[PosAngle={-135,-45,45,135,%s}](0,0){%s}(%s,0){%s}(%s,%s){%s}(%s,%s){%s}(%s,%s){%s}"
               %([round(angBAC-(180-angAEB)/2),round(angBAC+(angAEB)/2)][angAEB>90],A,AB,B,x_C,y_C,C,x_D,y_D,D,x_C/2,y_C/2,E))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkHashh}"%(A,E))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkHashh}"%(C,E))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkCros}"%(B,E))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkCros}"%(D,E))
    cor.append(u"\\pstLineAB{%s}{%s}\\pstLineAB{%s}{%s}\\pstLineAB{%s}{%s}\\pstLineAB{%s}{%s}"
               %(A,D,C,D,A,B,B,C))
    cor.append(cotation_h((0,0),(x_C/2,y_C/2),decimaux(AC/2),couleur="enonce"))
    cor.append(cotation_h((x_C/2,y_C/2),(AB,0),decimaux(BD/2),couleur="enonce"))
    cor.append(u"\\color{enonce}\\pstMarkAngle[linecolor=enonce]{%s}{%s}{%s}{%s\\degres}"%(A,E,B,angAEB))
    cor.append(u"\\end{pspicture}")

###################### LOSANGES ########################
    
def quest_losange_DD(exo,cor):
    """On donne un losange avec la longueur des deux diagonales"""
    #diagonale ou demi-diagonale
    A,B,C,D,E=geo.choix_points(5)
    nom=shuffle_nom([A,B,C,D])
    BD=0.2*random.randint(15,25)#AB mesure entre 4cm et 8cm, tracé horizontalement
    AC=0.2*random.randint(20,40)#AC mesure entre 4cm et 8cm, tracé horizontalement
    
    exo.append(u"\\item Trace un losange $%s$  tel que $%s%s=\\unit[%s]{cm}$ et $%s%s=\\unit[%s]{cm}$.\\par"
               %(nom,A,C,decimaux(AC),B,D,decimaux(BD)))
    cor.append(u"\\item Trace un losange $%s$  tel que $%s%s=\\unit[%s]{cm}$ et $%s%s=\\unit[%s]{cm}$.\\par"
               %(nom,A,C,decimaux(AC),B,D,decimaux(BD)))
    
    cor.append(u"Je note $%s$ le centre du losange.\\par"%E)
    
    cor.append("\\figureadroite{")
    cor.append(u"\\begin{pspicture}(%s,%s)(%s,%s)"%(-BD/2-0.7,-AC/2-0.4,BD/2+0.4,AC/2+0.4))
    cor.append(u"\\pstGeonode[PosAngle={-90,0,90,180}](0,%s){%s}(%s,0){%s}(0,%s){%s}(%s,0){%s}"
               %(-AC/2,A,BD/2,B,AC/2,C,-BD/2,D))
    cor.append("\\pstGeonode[PosAngle=-45](0,0){%s}"%E)
    
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkCross}"%(A,E))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkCross}"%(C,E))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkCros}"%(B,E))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkCros}"%(D,E))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}"%(A,B))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}"%(C,D))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}"%(B,C))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}"%(D,A))
    cor.append("\\pstRightAngle[linecolor=calcul]{%s}{%s}{%s}"%(B,E,C))
    cor.append(cotation_h((-BD/2,0),(0,0),decimaux(BD/2),couleur="enonce"))
    cor.append(cotation_h((0,-AC/2),(0,0),decimaux(AC/2),couleur="enonce"))
    cor.append(u"\\end{pspicture}}{")
    cor.append(u"Les diagonales du losange se coupent perpendiculairement en leur milieu~$%s$ ;  on a donc :"%E)
    cor.append(u"\\begin{enumerate}")
    cor.append(u"\\item $%s%s=%s%s=\\unit[%s]{cm}$ \\item $%s%s=%s%s=\\unit[%s]{cm}$ ;"
               %(A,E,C,E,decimaux(AC/2),B,E,E,D,decimaux(BD/2)))
    cor.append(u"\\item $(%s%s)\\perp(%s%s)$."%(A,C,B,D))
    cor.append("\\end{enumerate}}\n")

def quest_losange_CC(exo,cor):
    """On donne un losange avec la longueur d'un côté et un angle entre côtés"""
    #diagonale ou demi-diagonale
    A,B,C,D,E=geo.choix_points(5)
    nom=shuffle_nom([A,B,C,D])
    AB=0.2*random.randint(15,25)#AB mesure entre 4cm et 8cm, tracé horizontalement
    angBAD=random.randint(30,150)
    #Calcul pour tracer
    x_D=AB*cos(angBAD)
    y_D=y_C=AB*sin(angBAD)
    x_C=x_D+AB
    
    exo.append(u"\\item Trace un losange $%s$  tel que $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par"
               %(nom,A,B,decimaux(AB),B,A,D,angBAD))
    cor.append(u"\\item Trace un losange $%s$  tel que $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par"
               %(nom,A,B,decimaux(AB),B,A,D,angBAD))
    
    cor.append(u"Les quatre côtés du losange sont de même longueur donc $%s%s=%s%s=%s%s=%s%s=\\unit[%s]{cm}$ ;"%(A,B,B,C,C,D,D,A,decimaux(AB)))
    cor.append(u"\\begin{enumerate}")
    cor.append(u"\\item On trace le côté $[%s%s]$ puis on mesure l'angle $\\widehat{%s%s%s}=%s\\degres$ ;"%(A,B,B,A,D,angBAD))
    cor.append(u"\\item ensuite on reporte au compas les longueurs $%s%s$ et $%s%s$ pour construire le point $%s$."%(C,D,B,C,C))
    cor.append("\\end{enumerate}\n")
    cor.append(u"\\begin{pspicture}(%s,%s)(%s,%s)"%(min(0,x_D)-0.4,0-0.4,max(AB,x_C),y_D+0.4))
    cor.append(u"\\pstGeonode[PosAngle={%s,%s,%s,%s}](0,0){%s}(%s,0){%s}(%s,%s){%s}(%s,%s){%s}"
               %(angBAD/2-180,angBAD/2-90,angBAD/2,angBAD/2+90,A,AB,B,x_C,y_C,C,x_D,y_D,D))
    
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}"%(A,B))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}"%(C,D))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}"%(B,C))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}"%(D,A))
    
    cor.append(u"\\pstRotation[RotAngle=7,PointSymbol=none,PointName=none]{%s}{%s}[C_1]"%(D,C))
    cor.append(u"\\pstRotation[RotAngle=-7,PointSymbol=none,PointName=none]{%s}{%s}[C_2]"%(D,C))
    cor.append(u"\\pstArcOAB[linecolor=calcul]{%s}{C_2}{C_1}"%(D))
    cor.append(u"\\pstRotation[RotAngle=7,PointSymbol=none,PointName=none]{%s}{%s}[C_3]"%(B,C))
    cor.append(u"\\pstRotation[RotAngle=-7,PointSymbol=none,PointName=none]{%s}{%s}[C_4]"%(B,C))
    cor.append(u"\\pstArcOAB[linecolor=calcul]{%s}{C_4}{C_3}"%(B))
    cor.append(cotation((0,0),(AB,0),decimaux(AB),couleur="enonce"))
    cor.append(u"\\color{enonce}\\pstMarkAngle[linecolor=enonce]{%s}{%s}{%s}{%s\\degres}"%(B,A,D,angBAD))
    cor.append(u"\\end{pspicture}")
    
def quest_losange_CD(exo,cor):
    """On donne un losange avec la longueur d'un côté et la mesure d'un angle entre un côté et une diagonale"""
    
    A,B,C,D=geo.choix_points(4)
    nom=shuffle_nom([A,B,C,D])
    AC=0.1*random.randint(40,60)#AB mesure entre 4cm et 7cm, tracé horizontalement
    angBAC=random.randint(25,75)
    #Calcul pour tracer
    x_D=AC/2
    y_D=AC/2*tan(angBAC)
    
    exo.append(u"\\item Trace un losange $%s$ tel que $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par"
               %(nom,A,C,decimaux(AC),B,A,C,angBAC))
    cor.append(u"\\item Trace un losange $%s$ tel que $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par"
               %(nom,A,C,decimaux(AC),B,A,C,angBAC))
    #Rédaction
    cor.append(u" Comme $%s$ est un losange, je sais que $\\widehat{%s%s%s}=\\widehat{%s%s%s}=\\widehat{%s%s%s}=\\widehat{%s%s%s}=%s\\degres$."
               %(nom,B,A,C,A,C,B,A,C,D,C,A,D,angBAC))
    #Programme de construction
    cor.append(u"\\begin{enumerate}")
    cor.append(u"\\item Je trace le segment $[%s%s]$ mesurant $\\unit[%s]{cm}$ ;"%(A,C,decimaux(AC)))
    cor.append(u"\\item je trace $\\widehat{%s%s%s}$ et $\\widehat{%s%s%s}$ pour construire le point $%s$ ;"%(B,A,C,A,C,B,B))
    cor.append(u"\\item je trace $\\widehat{%s%s%s}$ et $\\widehat{%s%s%s}$ pour construire le point $%s$ ;"%(A,C,D,C,A,D,D))
    cor.append("\\end{enumerate}\n")
    cor.append(u"\\begin{pspicture}(-0.4,%s)(%s,%s)"%(-y_D-1,AC+0.4,y_D+1))
    #figure
    cor.append(u"\\pstGeonode[PosAngle={-180,0,90,-90}](0,0){%s}(%s,0){%s}(%s,%s){%s}(%s,%s){%s}"%(A,AC,C,x_D,y_D,D,x_D,-y_D,B))
    cor.append(u"\\pstLineAB[nodesepB=-1]{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}"%(A,B))
    cor.append(u"\\pstLineAB[nodesepB=-1]{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}"%(C,D))
    cor.append(u"\\pstLineAB[nodesepA=-1]{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}"%(B,C))
    cor.append(u"\\pstLineAB[nodesepA=-1]{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}"%(D,A))
    cor.append(u"\\pstLineAB{%s}{%s}"%(A,C))
    cor.append(cotation_h((0,0),(AC,0),decimaux(AC),couleur="enonce"))
    cor.append(u"\\color{enonce}\\pstMarkAngle[linecolor=enonce,Mark=MarkHash]{%s}{%s}{%s}{%s\\degres}"%(B,A,C,angBAC))
    cor.append(u"\\color{enonce}\\pstMarkAngle[linecolor=calcul,Mark=MarkHash]{%s}{%s}{%s}{}"%(C,A,D))
    cor.append(u"\\color{enonce}\\pstMarkAngle[linecolor=calcul,Mark=MarkHash]{%s}{%s}{%s}{}"%(D,C,A))
    cor.append(u"\\color{enonce}\\pstMarkAngle[linecolor=calcul,Mark=MarkHash]{%s}{%s}{%s}{}"%(A,C,B))
    cor.append(u"\\end{pspicture}")

def quest_losange_CDbis(exo,cor):
    """On donne un losange avec la longueur d'un côté et la mesure d'un angle entre un côté et une diagonale"""
    
    A,B,C,D=geo.choix_points(4)
    nom=shuffle_nom([A,B,C,D])
    AC=0.1*random.randint(40,60)#AB mesure entre 4cm et 7cm, tracé horizontalement
    angCDA=2*random.randint(15,70)
    #Calcul pour tracer
    angCAD=(180-angCDA)/2
    x_D=AC/2
    y_D=AC/2*tan(angCAD)
    
    exo.append(u"\\item Trace un losange $%s$ tel que $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par"
               %(nom,A,C,decimaux(AC),C,D,A,angCDA))
    cor.append(u"\\item Trace un losange $%s$ tel que $%s%s=\\unit[%s]{cm}$ et $\\widehat{%s%s%s}=%s\\degres$.\\par"
               %(nom,A,C,decimaux(AC),C,D,A,angCDA))

    #Rédaction des calculs
    cor.append(u"Les quatre côtés du losange sont de même longueur donc $%s%s=%s%s=%s%s=%s%s$.\\par"%(A,B,B,C,C,D,D,A))
    cor.append(u"Ainsi, le triangle $%s%s%s$ est isocèle en $%s$ et je peux calculer la mesure des angles $\\widehat{%s%s%s}=\\widehat{%s%s%s}$.\\par"
               %(A,C,D,A,A,C,D,C,A,D))
    cor.append(u"Dans un triangle, la somme des angles du triangle est égale à 180\\degres\\\\")
    cor.append(u"donc $\\widehat{%s%s%s}=\\widehat{%s%s%s}=(180\\degres-%s)\\div2=%s\\degres$"%(A,C,D,C,A,D,angCDA,angCAD))
    #Programme de construction
    cor.append(u"\\begin{enumerate}")
    cor.append(u"\\item Je trace le segment $[%s%s]$ mesurant $\\unit[%s]{cm}$ ;"%(A,C,decimaux(AC)))
    cor.append(u"\\item je trace $\\widehat{%s%s%s}$ et $\\widehat{%s%s%s}$ pour construire le point $%s$ ;"%(B,A,C,A,C,B,B))
    cor.append(u"\\item je trace $\\widehat{%s%s%s}$ et $\\widehat{%s%s%s}$ pour construire le point $%s$ ;"%(A,C,D,C,A,D,D))
    cor.append("\\end{enumerate}\n")
    
    cor.append(u"\\begin{pspicture}(-0.4,%s)(%s,%s)"%(-y_D-1,AC+0.4,y_D+1))
    #Figure
    cor.append(u"\\pstGeonode[PosAngle={-180,0,90,-90}](0,0){%s}(%s,0){%s}(%s,%s){%s}(%s,%s){%s}"%(A,AC,C,x_D,y_D,D,x_D,-y_D,B))
    cor.append(u"\\pstLineAB{%s}{%s}"%(A,C))
    cor.append(u"\\pstLineAB[nodesepB=-1]{%s}{%s}"%(A,B))
    cor.append(u"\\pstLineAB[nodesepB=-1]{%s}{%s}"%(C,B))
    cor.append(u"\\pstLineAB[nodesepB=-1]{%s}{%s}"%(C,D))
    cor.append(u"\\pstLineAB[nodesepB=-1]{%s}{%s}"%(A,D))
    #codage
    cor.append(u"\\color{calcul}\\pstMarkAngle[linecolor=calcul,Mark=MarkHash]{%s}{%s}{%s}{}"%(B,A,C))
    cor.append(u"\\color{calcul}\\pstMarkAngle[linecolor=calcul,Mark=MarkHash]{%s}{%s}{%s}{%s\\degres}"%(C,A,D,angCAD))
    cor.append(u"\\color{calcul}\\pstMarkAngle[linecolor=calcul,Mark=MarkHash]{%s}{%s}{%s}{%s\\degres}"%(D,C,A,angCAD))
    cor.append(u"\\color{calcul}\\pstMarkAngle[linecolor=calcul,Mark=MarkHash]{%s}{%s}{%s}{}"%(A,C,B))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}"%(A,B))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}"%(C,B))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}"%(C,D))
    cor.append(u"\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}"%(A,D))
    cor.append(cotation((0,0),(AC,0),decimaux(AC),couleur="enonce"))
    cor.append(u"\\color{enonce}\\pstMarkAngle[linecolor=enonce]{%s}{%s}{%s}{%s\\degres}"%(A,D,C,angCDA))
    cor.append(u"\\end{pspicture}")

################### CARRÉ #######################

def carre_diag(exo,cor):
    """trace un carré dont on donne la longueur de la diagonale"""

    A,B,C,D,E=geo.choix_points(5)
    nom=shuffle_nom([A,B,C,D])
    BD=AC=0.2*random.randint(20,40)#AC mesure entre 4cm et 8cm, tracé horizontalement
    
    exo.append(u"\\item Trace un carré $%s$  tel que $%s%s=\\unit[%s]{cm}$.\\par"
               %(nom,A,C,decimaux(AC)))
    cor.append(u"\\item Trace un carré $%s$  tel que $%s%s=\\unit[%s]{cm}$.\\par"
               %(nom,A,C,decimaux(AC)))

    cor.append(u"Je note $%s$ le centre du carré.\\par"%(E))
    cor.append(u" Les diagonales du carré se coupent perpendiculairement en leur milieu $%s$ donc on a :"%E)
    cor.append("\\begin{enumerate}")
    cor.append(u"\\item $(%s%s)\\perp(%s%s)$."%(A,C,B,D))
    cor.append(u"\\item  $%s%s=%s%s=%s%s=%s%s=\\unit[%s]{cm}$ ;"
               %(A,E,C,E,B,E,D,E,decimaux(BD/2)))
    cor.append("\\end{enumerate}\n")
    
    cor.append("\\begin{pspicture}(%s,%s)(%s,%s)"%(-BD/2-0.4,-AC/2-0.4,BD/2+0.4,AC/2+0.4))
    cor.append("\\pstGeonode[PosAngle={-90,0,90,180}](0,%s){%s}(%s,0){%s}(0,%s){%s}(%s,0){%s}"
               %(-AC/2,A,BD/2,B,AC/2,C,-BD/2,D))
    cor.append("\\pstGeonode[PosAngle=-45](0,0){%s}"%E)
    
    cor.append("\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkCros}"%(A,E))
    cor.append("\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkCros}"%(C,E))
    cor.append("\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkCros}"%(B,E))
    cor.append("\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=calcul}\\MarkCros}"%(D,E))
    cor.append("\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}"%(A,B))
    cor.append("\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}"%(C,D))
    cor.append("\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}"%(B,C))
    cor.append("\\pstLineAB{%s}{%s}\\lput{:U}{\\psset{linecolor=enonce}\\MarkHashh}"%(D,A))
    cor.append("\\pstRightAngle[linecolor=calcul]{%s}{%s}{%s}"%(B,E,C))
    cor.append(cotation_h((-BD/2,0),(0,0),decimaux(BD/2),couleur="enonce"))
    cor.append(u"\\end{pspicture}")



# -*- coding: utf-8 -*-
from ..classes.Polynome import Polynome
from .Arithmetique import pgcd

from random import randint,randrange
from math import *


#--------------outils pour la class Polynome----------------------

def TeX_division(dividende,diviseur):
    '''renvoie une chaine de caractere TeX pour afficher la division en détail'''
    quotient=Polynome({0:0})
    texquotient,restera= dividende/diviseur
    reste=dividende
    diviseur_degre=diviseur.deg
    sauve=min(dividende.puiss+restera.puiss)
    longueur=dividende.degre_max-sauve
    string= "$$\\renewcommand\\arraycolsep{0cm}\n\\begin{array}{c"
    for i in range(longueur):
        string+= "c"
    string+= "|c}\n"
    string+=tab_print(dividende,longueur+1)
    string+= str(diviseur)+ "\\\\\n"
    string+= "\\cline{"+str(dividende.deg+2-sauve)+"-"+str(dividende.deg+2-sauve)+"}\n"
    i=0
    while reste.deg>=diviseur.deg:
        for k in range(i):
            string+= " &"
        ajout_quotient_deg=reste.deg-diviseur_degre
        facteur= reste.dictio[max(0,reste.deg)]/diviseur.dictio[diviseur.deg]
        ajout_quotient=Polynome({ajout_quotient_deg:facteur},var=dividende.var)
        soustrait_reste=ajout_quotient*diviseur
        string +=tab_print(soustrait_reste,diviseur_degre+1-sauve,parenthese=True)
        for k in range(longueur-i-diviseur.deg+sauve):
            string+= " & "
        if i ==0:
            string+= str(texquotient)
        string+= "\\\\\n"
        string+= "\\cline{"+str(i+1) +"-"+str(i+diviseur.deg+1-sauve) +"}\n"
        avant=reste.deg
        reste=reste-soustrait_reste
        delta=avant-reste.degre_max
        for k in range(i):
            string+= " &"
        i=i+delta
        prochain=1
        string +=tab_print(reste,min(diviseur.deg+1,reste.degre_max+1)+delta-sauve,debut=delta)
        #fait descendre les monome du dividende
        for k in range(longueur-i-diviseur.deg+sauve):
            string+= " & "
        string+= "\\\\ \n"
    string+= "\\end{array}\n$$"

    string+="\n On a $$"+dividende.TeX()+" = \\left(" + texquotient.TeX()+"\\right) \\times \\left("+diviseur.TeX()+"\\right)"
    if restera!=Polynome(0):
        if len(restera.puiss)==1 and restera[restera.deg]>0:#monome
            string +="+"+restera.TeX()
        else:
            string+="+\\left("+restera.TeX()+"\\right)"
    string+="$$"
    return string
def tab_print(polynome,longueur=0,parenthese=False,debut=0):
    '''utilisé par TeX_division pour décaler le reste dans la partie gauche'''
    degre=polynome.degre_max+debut
    string=''
    if parenthese:
        string = "-("
        fin=")"
    else:
        fin=""
    if polynome==0:
        string += "+"+str(0)+ " &"
    else:
        for i in range(longueur):
            k=degre-i
            coeff=polynome.dictio.get(k,0)
            if coeff>=0:
                string+= "+"
            string += str(coeff)
            if k!=0:
                string+= polynome.var
            if k != 1 and k!=0:
                string+= u"^" + str(k)
            if longueur-i==1:
                    string+= fin
            string+= " & "
    return string

########################################################
#
#  construction de polynôme de degré 2
#  X est un Polynome
#  X=Polynome("x","x") donnera un polynome de degré 2
#  X=Polynome("x^2",x) donnera un polynôme bicarrée
#
########################################################
def poly_racines_quelconques(abs_a,abs_b,abs_c,X):
    '''renvoie un polynome de degré 2'''
    '''abs_a,abs_b,abs_c sont des entiers positifs majorant les valeurs absolues de a, b ,c'''
    
    a3=(2*randrange(2)-1)*randrange(1,abs_a+1)
    b3=randrange(abs_b)
    c3=randrange(-abs_c,abs_c)
    return a3*X**2+b3*X+c3
def poly_racines_fractionnaires(rac_min,rac_max,denom1,X):
    '''renvoie un polynome de degré2 à racines fractionnaires '''
    '''les racines sont comprises entre rac_min et rac_max'''
    '''denom1 majore le dénominateur des racines'''
    while 1:
        #pour éviter P=77x^2
        a2=2*randrange(2)-1 #a2=-1 ou 1
        p2facteur=[randint(1,denom1)*X-randint(rac_min,rac_max)for i in range(2)]
        pol2=a2*p2facteur[0]*p2facteur[1]
        if pol2[1]!=0 and pol2[0]!=0:
            break
    pol2=pol2.simplifie()
    simplifie=abs(pgcd(pgcd(int(pol2[0]),int(pol2[1])),int(pol2[2])))
    pol2= pol2/simplifie
    return pol2.simplifie()
def poly_racines_entieres(rac_min,rac_max,X,a1=1):
    
    while 1:
        p1facteur=[X-randrange(rac_min,rac_max) for i in range(2)]
        pol1=a1*p1facteur[0]*p1facteur[1]
        if pol1[1]!=0 or pol1[1]!=0:
            break
    return pol1

def poly_id_remarquables(rac_min,rac_max,X):
    '''Renvoie un polynome obtenu par une identité remarquable'''
    '''et sgns 
         -2 => (a-b)²
         +2 => (a+b)²
          0 => (a-b)(a+b)'''
    a=randint(1,10)
    while 1:
        coeff=randrange(rac_min,rac_max)
        racine=randrange(rac_min,rac_max)
        if coeff!=0 and racine!=0:
            break
    sgns=[[-1,1][randrange(2)]for i in range(2)]
    p1facteur=[coeff*X+sgns[i]*racine for i in range(2)]
    return a*p1facteur[0]*p1facteur[1],sum(sgns)   
        #sum(sgns) permet de connaître l'identité remarquable
        # -2 => (a-b)²
        # +2 => (a+b)²
        # 0 => (a-b)(a+b)

 #----------polynome de degré 3-------------
def poly_degre3_racines_entieres(rac_min,rac_max,X,racines=[-2,-1,0,1,2]):
    racine_evidente=racines[randrange(len(racines))]
    return (X-racine_evidente)*poly_racines_entieres(rac_min,rac_max,X)
def poly_degre3_racines_fractionnaires(rac_min,rac_max,denom1,X,racines=[-2,-1,0,1,2]):
    racine_evidente=racines[randrange(len(racines))]
    return (X-racine_evidente)*poly_racines_fractionnaires(rac_min,rac_max,denom1,X)
def poly_degre3_racines_quelconques(abs_a,abs_b,abs_c,X,racines=[-2,-1,0,1,2]):
    racine_evidente=racines[randrange(len(racines))]
    return (X-racine_evidente)*poly_racines_quelconques(abs_a=1,abs_b=10,abs_c=10,X=X)





# -*- coding: utf-8 -*-
import os,sys
sys.path.append("/home/nicolas/pyrogit/pyromaths")
from random import randrange
from ..outils.Polylycee import *

def genere_points(xmin,xmax,ymin,ymax,simple=False):

    if simple:#génère 3 intervalle de variation /\/ ou \/\
        if randrange(1):
            variation=[0,1,0]
        else:
            variation=[1,0,1]
        longueur=xmax-xmin
        hauteur=ymax-ymin
        x_variation=[xmin]
        x_variation.append(x_variation[0]+randrange(int(0.25*longueur),int(0.4*longueur)))
        x_variation.append(x_variation[1]+randrange(int(0.25*longueur),int(0.4*longueur)))
        x_variation.append(xmax)
        liste_points=[(x_variation[0],randrange(ymin+(1-variation[0])*int(0.4*hauteur),ymax-variation[0]*int(0.4*hauteur)))]
        for i in [1,2,3]:
            if variation[i-1]:
                liste_points.append((x_variation[i],randrange(liste_points[i-1][1],ymax)))
            else:
                liste_points.append((x_variation[i],randrange(ymin,liste_points[i-1][1])))
    else:
        liste_points=[(xmin,0)]
        for i in range(10):
            liste_points.append((liste_points[i][0]+randrange(2,5),randrange(ymin,ymax)))
    return liste_points

def bezier(simple=False,xmin=-15,xmax=15,ymin=-12,ymax=12):
    liste_points=genere_points(simple=simple,xmin=xmin,xmax=xmax,ymin=ymin,ymax=ymax)
    variation=[]
    largeur=liste_points[-1][0]-liste_points[0][0]
    points=[liste_points[0]]
    for i in range(1,len(liste_points)):
        if liste_points[i-1][1]<liste_points[i][1]:
            points.append(liste_points[i])
        elif liste_points[i-1][1]>liste_points[i][1]:
            points.append(liste_points[i])
    deltaxx=0.3*(points[1][0]-points[0][0])
    nderiv=float(points[1][1]-points[0][1])/(points[1][0]-points[0][0])
    apres=[(points[0][0]+0.5,points[0][1]+deltaxx*nderiv)]
    avant=[]
    if points[0][1]<points[1][1]:
        variation+='+'
    else:
        variation+='-'
    x_variation=[points[0][0]]
    y_variation=[points[0][1]]

    for i in range(1,len(points)-1):
        if (points[i][1]<=points[i-1][1] and points[i][1]<=points[i+1][1]):
            nderiv=0
            x_variation.append(points[i][0])
            y_variation.append(points[i][1])
            variation+='+'
        elif (points[i][1]>=points[i-1][1] and points[i][1]>=points[i+1][1]):
            nderiv=0
            x_variation.append(points[i][0])
            y_variation.append(points[i][1])
            variation+='-'
        else:
            nderiv=float(points[i-1][1]-points[i+1][1])/(points[i-1][0]-points[i+1][0])
        deltax=0.3*(points[i-1][0]-points[i][0])
        deltaxx=0.3*(points[i+1][0]-points[i][0])
        avant.append((points[i][0]+deltax,points[i][1]+deltax*nderiv))
        apres.append((points[i][0]+deltaxx,points[i][1]+deltaxx*nderiv))
    avant.append((points[len(points)-1][0]-0.5,points[len(points)-1][1]-0))
    string=str(points[0])
    for i in range(len(points)-1):
        string+=str(apres[i])+str(avant[i])+str(points[i+1])
    textbezier="\\input{tabvar}\n\
                \psset{unit="+str(14.0/largeur)+"cm}\n\
                \\begin{pspicture}"
    textbezier+="("+str(xmin)+","+str(ymin)+")("+str(liste_points[-1][0])+","+str(ymax)+")\n"
    textbezier+="\psgrid[subgriddiv=0,griddots=10,gridlabels=0pt]\n\
                \psaxes[linewidth=1pt,%\n\
                ticks=none,%\n\
                %labels=none\n\
                Dx=2\n\
                ]{->}(0,0)"
    textbezier+="("+str(xmin)+","+str(ymin)+")("+str(liste_points[-1][0])+","+str(ymax)+")\n"
    impression=textbezier+"\psbezier"+string+"\n"+"\\end{pspicture}\n\\vspace{2cm}\n\n"
    impression+="$$\n\
                \\tabvar{%\n\
                \\tx{x}&\\tx{"+str(x_variation[0])+"}"
    for i in range(1,len(x_variation)):
        impression+="&& \\tx{"+str(x_variation[i])+"}"
    impression+="&& \\tx{"+str(points[-1][0])+"}\\cr\n"
    impression+="\\tx{f(x)}"
    for i in range(len(variation)):
        if variation[i]=='+':
            impression+="&\\txb{"+str(y_variation[i])+"}&\\fm"
        else:
            impression+="&\\txh{"+str(y_variation[i])+"}&\\fd"
    if variation[-1]=='+':
        impression+="&\\txh{"+str(points[-1][1])+"}"
    else:
        impression+="&\\txb{"+str(points[-1][1])+"}"
    return impression+"\\cr\n}$$\n\n"
        

def tab_var(fonc,Intervalle=["-\\infty","+\\infty"]):
    if isinstance(fonc,Polynome):
        return tab_var_poly(fonc,Intervalle[0],Intervalle[1])
    else:
        return "Je ne sais pas étudier les variations de cette fonction"

def tab_var_poly(P,borneinf="-\\infty" , bornesup=u"+\\infty"):
    Intervalle=[borneinf,bornesup]
    #print"borneinf=",borneinf
    tab_var="\\input{tabvar}\n"
    tab_var+="$$\\tabvar{%\n"
    if P.deg==0:
        tab_var+=tabvar_x(Intervalle,P.var)
        Cste=str(P(0))
        tab_var+="\\tx{f("+P.var+")}&\\tx{"+Cste+"}&\\fhm&\\tx{"+Cste+"}\\cr\n"
        return tab_var+"}$$\n"
    elif P.deg==1:
        tab_var+=tabvar_x(Intervalle,P.var)
        if P.dictio[1]>0:
            tab_var+="\\tx{f("+P.var+")}& &\\fm& \\cr\n"
        else:
           tab_var+="\\tx{f("+P.var+")}& &\\fd&\\cr\n"
        return tab_var+"}$$\n"
    elif P.deg==2:
        alpha=-P.dictio[1]/(2*P.dictio[2])
        extremum=P(alpha)
        if (borneinf=="-\\infty" or Intervalle[0]<alpha) and\
           (Intervalle[1]=="+\\infty" or Intervalle[1]>alpha):
            listex=[Intervalle[0],alpha,Intervalle[1]]
            if P.dictio[0]>0:
                tab_var_y ="\\tx{f("+P.var+")}& &\\fd&\\txb{"+nombre_TeX(extremum)+"}&\\fm&\\cr\n"
            else:
                tab_var_y ="\\tx{f("+P.var+")}& &\\fm&\\txh{"+nombre_TeX(extremum)+"}&\\fd&\\cr\n"     
        else:
            listex=Intervalle
            if P.dictio>0:
                if not(borneinf=="-\\infty") and Intervalle[0]>alpha:
                    tab_var_y="\\tx{f("+P.var+")}& &\\fm& \\cr\n"
                else:
                    tab_var_y="\\tx{f("+P.var+")}& &\\fd&\\cr\n"
            else:
                if not(borneinf=="-\\infty") and Intervalle[0]>alpha:
                    tab_var_y="\\tx{f("+P.var+")}& &\\fd&\\cr\n"
                else:
                    tab_var_y="\\tx{f("+P.var+")}& &\\fm& \\cr\n"
        tab_var+=tabvar_x(listex,P.var)
        tab_var+=tab_var_y
        return tab_var+"}$$\n"
    elif P.deg==3:
        return u"attends un peu pour le degré 3"
        ##Les fonctions développées pour les équations du second degré doivent aider
    else:
        return u"Degré trop élevé"
            ##Cas partidulier où P' a une racine évidente, ou polynome bicarré
        

def tabvar_x(listex,var):
    '''imprime la premiere ligne du tabvar'''
    var_x="\\tx{"+var+"}&\\tx{"+nombre_TeX(listex[0])+"}"
    for i in range(1,len(listex)):
        var_x+="&&\\tx{"+nombre_TeX(listex[i])+"}"
    return var_x+"\\cr\n"

def tabvar_de_f(listef):
    '''inutile'''
    return ""
def nombre_TeX(nombre):#A remplacer par sepmillier
    if type(nombre) in [type(int),type(float)]:
        return str(nombre)
    elif isinstance(nombre,Fractions):
        if nombre.denominateur==1:
            return nombre_TeX(nombre.numerateur)
        else:
            return "\\dfrac{"+str(nombre.numerateur)+"}{"+str(nombre.denominateur)+"}"
    else:
        return str(nombre)

if __name__=="__main__":
    from TEST.imprimetest import *
    imprime_TeX(bezier(),"bezier.tex")
    P=Polynome("x^2+5x-6")
    imprime_TeX(tab_var(P),"polyvar.tex")
    



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#
#----------------------------------------------------------------------
# Pyromaths : Initiation au calcul littéral
#----------------------------------------------------------------------
from ..outils import Arithmetique
from ..outils.Affichage import tex_coef
from ..outils.Priorites3 import texify, priorites
import random

def valeurs_reduire():
    """Travail sur les bases du calcul littéral en quatrième"""
    var = "abcdfghkmnpqrstuvwxyz"
    var = var[random.randrange(len(var))]
    op = "+*-*"[random.randrange(4)]
    if op == "*":
        deg1 = random.randrange(3)
        if deg1 == 2: deg2=0
        elif deg1 == 1: deg2 = random.randrange(2)
        else: deg2 = random.randrange(1, 3)
    else:
        deg1 = random.randrange(1, 3)
        deg2 = [i for i in range(3)]
        deg2.extend([deg1]*7)
        random.shuffle(deg2)
        deg2 = deg2.pop(random.randrange(len(deg2)))
    a1, a2 = 0, 0
    while not a1 or not a2:
        a1 = random.randrange(-10, 11)
        a2 = random.randrange(-10, 11)
    p1 = "Polynome(\"%s%s^%s\")" % (a1, var, deg1)
    p2 = "Polynome(\"%s%s^%s\")" % (a2, var, deg2)
    return p1 + op + p2

def reduire():
    """Travail sur les bases du calcul littéral en quatrième"""
    exo = ["\\exercice", u"Réduire, si possible, les expressions suivantes :",
           "\\begin{multicols}{3}\\noindent", "  \\begin{enumerate}"]
    cor = ["\\exercice*", u"Réduire, si possible, les expressions suivantes :",
           "\\begin{multicols}{3}\\noindent", "  \\begin{enumerate}"]
    for i in range(9):
        a = valeurs_reduire()
        solve = [a]
        exo.append("\\item $\\thenocalcul = " + texify(solve)[0] + "$")
        cor.append("\\item $\\thenocalcul = " + texify(solve)[0] + "$")
        solve = priorites(a)
        solve.insert(0, a)
        solve = texify(solve)
        if len(solve)>1:
            for e in solve[1:]:
                cor.append("\\[\\thenocalcul = "  + e + "\\]")
        exo.append("\\stepcounter{nocalcul}")
        cor.append("\\stepcounter{nocalcul}")
    exo.extend(["  \\end{enumerate}", "\\end{multicols}"])
    cor.extend(["  \\end{enumerate}", "\\end{multicols}"])
    return (exo, cor)

def valeurs_reduire_somme():
    """Réduire une somme de six monômes de degrés 0, 1 et 2"""
    import random
    var = "abcdfghkmnpqrstuvwxyz"
    var = var[random.randrange(len(var))]
    l=[]
    for i in range(6):
        a = 0
        degre = i//2
        while not a:
            a = random.randrange(-10, 11)
        l.append("Polynome(\"%s%s^%s\")" % (a, var, degre))
    random.shuffle(l)
    t = l[0]
    for i in range(1, len(l)):
        t += "+-"[random.randrange(2)] + l[i]
    return t

def valeurs_reduire_sommeprod():
    """Réduire une expression de six monômes de degrés 0, 1 et 2"""
    import random
    var = "abcdfghkmnpqrstuvwxyz"
    var = var[random.randrange(len(var))]
    l=[]
    for i in range(5):
        a = 0
        degre = i//2
        while not a:
            a = random.randrange(-10, 11)
        l.append("Polynome(\"%s%s^%s\")" % (a, var, degre))
    random.shuffle(l)
    t = l[0]
    for i in range(1, len(l)):
        if l[i-1][-3] in "01" and l[i][-3] in "01": t += "*" + l[i]
        else: t += "+-"[random.randrange(2)] + l[i]
    return t

def reduire_expressions():
    """Travail sur les bases du calcul littéral en quatrième"""
    exo = ["\\exercice", u"Réduire les expressions littérales suivantes :",
           "\\begin{multicols}{2}\\noindent", "  \\begin{enumerate}"]
    cor = ["\\exercice*", u"Réduire les expressions littérales suivantes :",
           "\\begin{multicols}{2}\\noindent", "  \\begin{enumerate}"]
    for i in range(6):
        if i < 3: a = valeurs_reduire_somme()
        else: a = valeurs_reduire_sommeprod()
        solve = [a]
        exo.append("\\item $\\thenocalcul = " + texify(solve)[0] + "$")
        cor.append("\\item $\\thenocalcul = " + texify(solve)[0] + "$")
        solve = priorites(a)
        solve.insert(0, a)
        solve = texify(solve)
        if len(solve)>1:
            for e in solve[1:]:
                cor.append("\\[\\thenocalcul = "  + e + "\\]")
        exo.append("\\stepcounter{nocalcul}")
        cor.append("\\stepcounter{nocalcul}")
    exo.extend(["  \\end{enumerate}", "\\end{multicols}"])
    cor.extend(["  \\end{enumerate}", "\\end{multicols}"])
    return (exo, cor)




# -*- coding: utf-8 -*-

import sys, os, codecs
from ..classes.SecondDegre import Poly2
from ..outils.Affichage import printlist
from ..outils.Polynomes import choix_coeffs
from ..outils.Arithmetique import *

def exo_second_degre():
    """Créer une exercice comprenant des équations et inéquations du second degré. PAS FINI."""
    exo = "\\exercice\n\
           \\begin{multicols}{2}\n\
           \\noindent "+ u"Question 1 : Chercher les racines des polynômes suivants :\\par\n"
    cor = "\\exercice*\n\
           \\begin{multicols}{2}\n\ \\noindent "+ u"Question 1 : Chercher les racines des polynômes suivants :\\par\n"

    ## Equations
    pol1 = choix_coeffs(False, 2, True)
    pol2 = choix_coeffs(False, 1, False)
    pol3 = choix_coeffs(False, 0, True)
    pol4 = choix_coeffs(False, 2, False)

    ## Inequations
    pol5 = choix_coeffs(True, 0, True)
    pol6 = choix_coeffs(False, 2, True)
    pol7 = choix_coeffs(False, 1, False)
    pol8 = choix_coeffs(False, 2, False)

    ## Somme et produit des racines
    pol9 = choix_coeffs(True, 2, False)

    ## Problème du rectangle
    pol10 = choix_coeffs(True, 2, False)

    exo+="\\par a)$" + pol1[1].print_signe("=")+"$\n"
    exo+="\\par b)$" + pol2[1].print_signe("=")+"$\n"
    exo+="\\par c)$" + pol3[1].print_signe("=")+"$\n"
    exo+="\\par d)$" + pol4[1].print_signe("=")+"$\n"

    cor+="\\par a)$" + pol1[1].print_signe("=")+"$\n"
    cor+="\\par b)$" + pol2[1].print_signe("=")+"$\n"
    cor+="\\par c)$" + pol3[1].print_signe("=")+"$\n"
    cor+="\\par d)$" + pol4[1].print_signe("=")+"$\n"

    cor+="\\par a) On calcule le discriminant : $\\Delta = " + str(pol1[1].delta) + "$.\n"
    cor+="\\par Comme le discrimimant est $" + pol1[1].signedelta + u"$, on en déduit que ce polynôme possède $" + str(pol1[1].nbrac) + "$ racines."

    exo+=u"\\par Question 2 : Résoudre les inéquations suivantes : "
    cor+=u"\\par Question 2 : Résoudre les inéquations suivantes : "

    exo+="\\par a)$" + pol5[1].print_signe("\\le")+"$\n"
    exo+="\\par b)$" + pol6[1].print_signe("\\ge")+"$\n"
    exo+="\\par c)$" + pol7[1].print_signe(">")+"$\n"
    exo+="\\par d)$" + pol8[1].print_signe("<")+"$\n"

    cor+="\\par a)$" + pol5[1].print_signe("\\le")+"$\n"
    cor+="\\par b)$" + pol6[1].print_signe("\\ge")+"$\n"
    cor+="\\par c)$" + pol7[1].print_signe(">")+"$\n"
    cor+="\\par d)$" + pol8[1].print_signe("<")+"$\n"

    exo+="\\par Question 3 : Trouver deux nombres sachant que leur somme est " + str(pol9[1].b) + " et leur produit " + str(pol9[1].c) + ".\n"
    cor+="\\par Question 3 : Trouver deux nombres sachant que leur somme est " + str(pol9[1].b) + " et leur produit " + str(pol9[1].c) + ".\n"

    exo+=u"\\par Question 4 : Trouver les dimensions d'un rectangle, sachant que son périmètre est " + str(pol9[1].b) + " et son aire " + str(pol9[1].c) + "."
    cor+=u"\\par Question 4 : Trouver les dimensions d'un rectangle, sachant que son périmètre est " + str(pol9[1].b) + " et son aire " + str(pol9[1].c) + "."

    exo+="\\end{multicols}\n"
    cor+="\\end{multicols}\n"
    return (exo, cor)

if __name__=="__main__":
    from TEST.imprimetest import *
    exo,cor=exo_second_degre()
    imprime_TeX(exo+"\\pagebreak"+cor,"secondegre.tex")



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

from sys import argv, exit
import sys
from os import access, R_OK, makedirs, chdir
from os.path import join, isdir, dirname, realpath, split
from codecs import open

def main():
#===============================================================================
# Imports spécifiques à Pyromaths
#===============================================================================
    from outils.System import create_config_file, modify_config_file
    from outils.TestEnv import test
    from Values import CONFIGDIR

    import interface
    from PyQt4 import QtGui, QtCore
    class StartQT4(QtGui.QMainWindow, interface.Ui_MainWindow):
        def __init__(self, parent=None):
            QtGui.QWidget.__init__(self, parent)
            self.ui = interface.Ui_MainWindow()
            self.ui.setupUi(self)

    #===========================================================================
    # Création du fichier de configuration si inexistant
    #===========================================================================
    if not access(join(CONFIGDIR, "pyromaths.xml"), R_OK):
        if not isdir(CONFIGDIR): makedirs(CONFIGDIR)
        f = open(join(CONFIGDIR, "pyromaths.xml"), encoding='utf-8', mode='w')
        f.write(u"" + create_config_file())
        f.close()
    modify_config_file(join(CONFIGDIR, "pyromaths.xml"))
    templatesdir = join(CONFIGDIR,  "templates")
    if not isdir(templatesdir): makedirs(templatesdir)
    packagesdir = join(CONFIGDIR,  "packages")
    if not isdir(packagesdir): makedirs(packagesdir)

    app = QtGui.QApplication(argv)
    pyromaths = StartQT4()

    pyromaths.show()
    test(pyromaths)

    exit(app.exec_())

if __name__ == "__main__":
    basedir = dirname(realpath(__file__))
    _path, _dir = split(basedir)
    sys.path[0] = realpath(_path)
    exec("from %s import pyromaths" % _dir)
    pyromaths.main()



# -*- coding: utf-8 -*-


if __name__=="__main__":
    import sys,os
    sys.path.append(os.path.join('..'))

from .Racine import simplifie_racine, RacineDegre2, sqrt
from .Fractions import Fractions
import re
from ..outils.Affichage import pTeX, TeX, radicalTeX, fTeX, Fractions, tTeX

class Polynome:
    '''Classe de polynôme pour le lycee'''
    def __init__(self, liste_coeff, var = "x"):
        self.var = var # Lettre pour la var
        liste_reduite={}
        if isinstance(liste_coeff,str):
            liste_coeff=str_Polynome(liste_coeff,var)
        elif isinstance(liste_coeff,list):
            liste_coeff=dict((i,liste_coeff[i])for i in range(len(liste_coeff)))
        elif isinstance(liste_coeff,int) or isinstance(liste_coeff,Fractions.Fractions) or isinstance(liste_coeff,float):
            liste_coeff={0:liste_coeff}
        for i in liste_coeff.iterkeys():
            if liste_coeff[i] != 0:
                liste_reduite[i]=liste_coeff[i]+Fractions.Fractions(0, 1)
        if liste_reduite=={} or liste_coeff==[]:
            liste_reduite={0:0}
        self.dictio = liste_reduite
        self.puiss = liste_reduite.keys()
        self.puiss.sort(reverse=True)
        self.deg=self.degre()
        self.degre_max=max(0,self.deg)

    def __len__(self):
        return max(0,self.deg)+1
    
    def degre(self):
        degre=float("-inf")
        for i in self.dictio.iterkeys():
            if i > degre and self.dictio[i] !=0:
                degre=i
        return degre

    def __getitem__(self,i):
        '''P[i] renvoie le coefficient de rang i'''
        return self.dictio.get(i,0)
    
    def __str__(self):
        '''renvoie une str pour un affichage python'''
        return self.TeX(var=self.var)
    
           
    def TeX(self,var='',display=True,parenthese=False):
        '''renvoie une chaine de caractere imprimant les fractions dans TeX'''
        if var=='':
            var=self.var
        exposants = []+self.puiss
        premier = 1
        string=''
        if display:
            fractex="\\dfrac"
        else:
            fractex="\\frac"
        for exposant in exposants :
            if premier:
                if self[exposant]==1 and exposant!=0:
                    string=''
                elif self[exposant]==-1 and exposant!=0:
                    string='-'
                else:
                    string=TeX(self[exposant])
                premier=0
            elif self[exposant] != 1:
                if self.dictio[exposant] == -1:
                    string += '-'
                else:
                    string+=tTeX(self[exposant])
            else:
                string+="+"
            if exposant == 1:
                terme = var
            elif (exposant == 0) and ((self[exposant] == 1) or (self[exposant] == -1)):
                terme = '1'
            elif exposant == 0:
                terme = ''
            else:
                terme = var + u'^' + str(exposant)
            string+=terme
        if parenthese and (len(self.puiss)>1 or self[self.degre_max]<0):
            return "\\left("+string+"\\right)"
        else:
            return string

    def __add__(self, other):
        if isinstance(other,int) or isinstance(other,float) or  isinstance(other,Fractions.Fractions) or isinstance(other,RacineDegre2):
            return self+Polynome({0:other},var=self.var)
        result={0:0}
        liste1=self.puiss
        liste2=other.puiss
        for i in liste1 :
            s=self.dictio.get(i)+other.dictio.get(i,0)
            if s!=0:
                result[i]=s
        for i in liste2:
            if i not in liste1:
                result[i]=other.dictio[i]
        return Polynome(result,var=self.var)
    
    def __mul__(self, other):
        if isinstance(other,Polynome):
            result=Polynome({0:0},var=self.var)
            for i in self.dictio.iterkeys():
                for j in other.dictio.iterkeys():
                    exposant=i+j
                    coefficient=self.dictio[i]*other.dictio[j]
                    result = result + Polynome({exposant:coefficient},var=self.var)
            return result
        else:
            return self*Polynome(other)
    def __ne__(self,other):
        return not(self == other)
    def __eq__(self,other):
        if (isinstance(other,int) or isinstance(other,Fractions.Fractions) or isinstance(other,float)):
            if self.degre_max==0 and self[0]==other:
                return True
            else:
                return False
        elif self.var==other.var and self.dictio==other.dictio:
            return True
        else:
            return False
    
    def __pow__(self, other):
        if isinstance(other,int) and other>=0:
            result=Polynome({0:1},self.var)
            for i in xrange(other):
                result*=self
            return result

    def __radd__(self, other):
        if isinstance(other,int) or isinstance(other,float) or isinstance(other,Fractions.Fractions):
            return self+Polynome({0:other},var=self.var)
        return self + other

    def __sub__(self, other):
        return self + (-other)

    def __rsub__(self, other):
        return -(self - other)

    def __neg__(self):
        return self * Polynome({0:-1},var=self.var)

    def __rmul__(self,nombre):
        return self*Polynome({0:nombre})
    
    def __div__(self,other):
        if isinstance(other,int):
            return Fractions.Fractions(1,other)*self
        elif isinstance(other,Fractions.Fractions) or isinstance(other,float)or isinstance(other,RacineDegre2):
            return (1/other)*self
        else:
            quotient=Polynome({},var=self.var)
            reste=self
            diviseur_degre=other.deg
            while diviseur_degre <= reste.deg:
                ajout_quotient_deg=reste.deg-diviseur_degre
                facteur= reste.dictio[reste.deg]/other.dictio[other.deg]
                ajout_quotient=Polynome({ajout_quotient_deg:facteur},var=self.var)
                quotient=quotient+ajout_quotient
                soustrait_reste=ajout_quotient*other
                reste=reste-soustrait_reste
            return quotient,reste
    def simplifie(self):
        result={}
        for i in self.puiss:
            if isinstance(self[i],(Fractions.Fractions,RacineDegre2)):
                result[i]=self[i].simplifie()
            else:
                result[i]=self[i]
        return Polynome(result)
    
    def __call__(self,x,):
        '''renvoie la Fraction ou l'int P(x)'''
        if x==float("inf") or x==float("-inf"):
            if self.degre_max==0:
                return self[0]
            elif self.deg%2==0:
                return self[self.deg]*float("inf")
            else:
                return self[self.deg]*x
        elif isinstance(x,str):
            return self.TeX(var=x)
        else:
            result=0
            for i in self.dictio.iterkeys():
                result= result+self[i]*x**i
            if isinstance(result,Fractions.Fractions):
                result=result.simplifie()
            if result.denominateur==1:
                return result.numerateur
            else:
                return result

    def racine_evidente(self,liste=[-2,-1,0,1,2]):
        listerac=[]
        for i in liste:
            if self(i)==0:
                listerac=listerac+[i]
        return listerac

    def factorise(self,TeX=False,racines=[0,-1,1,2,-2]):
        facteurs=[Polynome({0:self.dictio[self.deg]},var=self.var)]
        developpe,reste=self/facteurs[0]
        for r in racines: 
            while developpe(r)==0:
                rac=-r
                developpe,reste = developpe/Polynome({1:1,0:rac},var=self.var)
                facteurs=facteurs+[Polynome({1:1,0:rac})]
        if TeX:
            stringTeX=""
            if not(facteurs[0]==Polynome({0:1},var=self.var)):
                stringTeX+=facteurs[0].TeX()
            for i in facteurs[1:]:
                if i[0]!=0:
                    stringTeX+="\\left("+i.TeX()+"\\right) \\times "
                else:
                    stringTeX+=i.TeX()+" \\times "
            if developpe==Polynome({0:1},var=self.var):
                return stringTeX[:-7]
            else:
                return stringTeX+"\\left("+developpe.TeX()+"\\right)"
        else:
            return facteurs + [developpe]

    def derive(self):
        result={}
        for i in self.dictio.iterkeys():
            if i == 0:
                result[0]=0
            else:
                result[i-1]=int(i)*self[i]
        return Polynome(result,self.var)
    def primitive(self):
        result={}
        for i in self.dictio.iterkeys():
            result[i+1]=Fractions.Fractions(1,int(i+1))*self.dictio[i]
        return Polynome(result,self.var)
    
def str_Polynome(string,var='x'):
    '''str -> dict'''
    #TODO reconnaitre les coefficients fractionnaires
    resultat = []
    termes = {}
    separeplus = (string).split("+")
    for element in separeplus:
        temp = []
        temp = element.split("-")
        for bis in temp:
            resultat.append(bis)
            if temp.index(bis) != len(temp)-1:
                resultat.append(-1)
        if separeplus.index(element) != len(separeplus)-1:
            resultat.append(1)
    if '' in resultat:
        resultat.remove('')
    for element in resultat:
        index = resultat.index(element)
        if index > 0:
            coeff = resultat[index-1]
        else:
            coeff = 1
        if (element not in [-1, 1]):
            if element==var:
                termes[1]=(coeff)
            else:
                a = re.findall('\d+(?:\.\d*)?', element)
                if (len(a) == 1) and (not re.findall('\^', element)) and (re.findall(var, element)):
                    termes[1]=coeff * Fractions.Fractions(eval(a[0]))
                elif (len(a) == 1) and (re.findall(var, element)):
                    termes[int(a[0])]=coeff * 1
                elif (len(a) == 1):
                    termes[0]=coeff * Fractions.Fractions(eval(a[0]))
                else:
                    termes[int(a[1])]=coeff * Fractions.Fractions(eval(a[0]))
    return termes

if __name__=="__main__":
    from TEST.imprimetest import *
    X=Polynome("x")
    P=Polynome({2:4,1:8,0:1,3:0})
##    Q=Polynome({2:-4,0:-1,1:8})
##    R=Polynome({3:1,2:1,1:-2})
##    D=Polynome({1:1,0:2})
##    F=Polynome("x-4")
##    FF=Polynome("x^2-3")
##    Divi=FF*F+7
##    FR=Polynome({3:Fractions(2,3),2:Fractions(2,3),1:Fractions(-4,3)})
##    print "P=", P
##    print "Q=", Q
##    print "R=", R
##    print "FR=",FR
##    R_facteur=R.factorise()
##    print "R=",R.factorise(TeX=True)
##    #TeX_division(R,D)
##    print Divi*Polynome("x^3")-28*X
##    #imprime_TeX(TeX_division(Divi*Polynome("x^3")-28*X,F*X))



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#


##- additions / soustractions de vecteurs ( difficulté : ne pas sortir du cadre )
##- multiplication d'un vecteur par un réel +  add + sous ( difficulté : ne pas sortir du cadre )
##- lire les coordonnées d'un vecteur
##- placer un point B connaissant A et les coordonnées de vec{AB} : avec coordonnées ou i et j ( difficulté : ne pas sortir du cadre )
##- calcul de norme
##- simplifier des sommes
##- problèmes de colinéarité

from ..classes.Vecteurs import randvect, Vecteur
from ..classes.Racine import simplifie_racine
import math
from random import randint, shuffle

def dist_bords(a,b):
    '''Calcule les distances minimales d'un point de coordonnées (a,b) aux bords du cadre, selon l'axe x et y.'''
    x = min(a,18-a) # la largeur vaut 18
    y = min(b,10-b) # la hauteur vaut 10
    return (x,y)

def pair(n):
    '''Retourne le plus petit entier pair strictement plus grand que n'''
    if (n % 2 == 0):
      return n+2
    else:
      return n+1

def AffNom(u,crd = 0):
    '''Renvoie les coordonnées pour l'affichage du nom du vecteur u.'''
    if u.x == 0 and math.fabs(u.y) > 2:
      coord = (0,u.y / 2)
    elif u.x == 0:
      coord = (-0.5,u.y / 2)
    elif u.y == 0 and math.fabs(u.x) > 2:
      coord = (u.x / 2, 0)
    elif u.y == 0:
      coord = (u.x / 2, - 0.5)
    elif math.fabs(u.x)+math.fabs(u.y) < 3:
      coord = (u.x/ 2.0 + 0.5,u.y / 2.0 + 0.5)
    else:
      coord = (u.x /2,u.y / 2)
    return str(coord[0]) + "," + str(coord[1])

def ChoixVecteur(u,v,w,x,y):
  listecoeff = [0.5, -0.5, -1, -2, 2, 3, -3]
  listevect = [(u,"u"),(v,"v"),(w,"w")]
  shuffle(listecoeff)
  shuffle(listevect)
  for vec in listevect:
    for coeff in listecoeff:
      if ( 0 <= x + coeff * vec[0].x <= 18 ) and ( 0 <= y + coeff * vec[0].y <= 10 ):
        return (coeff, coeff * vec[0], vec[1])



def repr_somme(u,v,u1,u2,cor,larg=0):
    '''Représente la somme des vecteurs u + v.'''

    a = u + v

    if (u.x * a.x >= 0 ):
      largeur = max(math.fabs(u.x),math.fabs(a.x))
      if ( a.x > 0 ):
         departx = 0
      elif ( a.x == 0 ):
        departx = -u.x/2.0+math.fabs(u.x)/2
      else:
         departx = max(math.fabs(u.x),math.fabs(a.x))
    else:
      largeur = math.fabs(u.x)+math.fabs(a.x)
      if ( u.x >= 0 ):
         departx = -a.x
      else:
         departx = -u.x
    if (u.y * a.y >= 0 ):
      hauteur = max(math.fabs(u.y),math.fabs(a.y))
      if ( a.y > 0 ):
         departy = 0
      elif ( a.y == 0 ):
       departy = -u.y/2.0+math.fabs(u.y)/2
      else:
         departy = max(math.fabs(u.y),math.fabs(a.y))
    else:
      hauteur = math.fabs(u.y)+math.fabs(a.y)
      if ( u.y >= 0 ):
         departy = -a.y
      else:
         departy = -u.y

    if int(larg) + largeur > 18:
      cor.append("\\par") # Figure trop large avec la précédente, il faut passer à une nouvelle ligne.

    depart = "(" + str(departx) + "," + str(departy) + ")"
    largeur = str(pair(int(largeur)))
    hauteur = str(pair(int(hauteur)))

    cor.append(u"\\begin{pspicture*}(0,0)(" + largeur + "," + hauteur + ")")
    cor.append(u"\\psgrid[subgriddiv=2, gridlabels=0pt]")
    cor.append(u"\\psset{unit=10mm,arrowscale=2}")

    cor.append(u"\\rput" + depart + "{")
    cor.append(u"\\psline[linewidth=1pt, linecolor=DarkGreen]{|->}(0, 0)(" + str(u.x) + ", " + str(u.y) + ")") ## Premier Vecteur
    cor.append(u"\\rput(" + AffNom(u) + ") \
          {\\psframebox[linecolor=white, fillcolor=white, fillstyle=solid]{\\textcolor{DarkGreen}{$\\overrightarrow{" + u1 + "}$}}}")

    cor.append(u"\\psline[linewidth=1pt, linecolor=DarkBlue]{|->}(" + str(u.x) + ", " + str(u.y) + ")(" + str(a.x) + ", " + str(a.y) + ")") ## 2e Vecteur
    k = Vecteur(u.x+a.x,u.y+a.y)
    cor.append(u"\\rput(" + AffNom(k) + ") \
          {\psframebox[linecolor=white, fillcolor=white, fillstyle=solid]{\\textcolor{DarkBlue}{$\\overrightarrow{" + u2 + "}$}}}")
    if len(u2)>1:
      sgn = "-"
    else:
      sgn = "+"
    cor.append(u"\\psline[linestyle=dashed, linewidth=1pt, linecolor=DarkRed]{|->}(0, 0)(" + str(a.x) + ", " + str(a.y) + ")") ## Résultat de l'opération
    cor.append(u"\\rput(" + AffNom(a) + ") \
          {\psframebox[linecolor=white, fillcolor=white, fillstyle=solid]{\\textcolor{DarkRed}{$\\overrightarrow{" + u1 + "}" + sgn + "\\overrightarrow{" + u2[-1] + "}$}}}")

    cor.append(u"}")
    cor.append(u"\\end{pspicture*}")
    return largeur ## récupérer la largeur pour éviter d'aligner des figures trop larges sur la feuille

def vecteurs_add():
    '''Exercice sur la définition des vecteurs et leurs sommes.'''
    t = None
    while not t:
    # Pour être sûr que l'exercice ait des solutions
        (u, posux, posuy) = randvect(0, 10)
        (v, posvx, posvy) = randvect(math.fabs(u.x)+1, 10)
        (w, poswx, poswy) = randvect(math.fabs(v.x)+math.fabs(u.x)+2, 10)

        ## Construction du point pour la question 2
        if 18 - poswx - max(w.x,0) > 0:
          restes = (18 - poswx - max(w.x,0),10)
          pointy = randint(0,10)
        elif poswy + min(w.y,0) > 10 - poswy - max(w.y,0):
          restes = (poswx+min(w.x,0),poswy + min(w.y,0))
          pointy = randint(0,restes[1])
        else:
          restes = (poswx+min(w.x,0),10 - poswy - max(w.y,0))
          pointy = randint(10 - restes[1],10)

        pointx = randint(18 - restes[0],18)

        t = ChoixVecteur(u,v,w,pointx,pointy)

    exo=["\\exercice"]
    cor=["\\exercice*"]

    exo.append(u"\\begin{pspicture*}(0,0)(18,10)")
    exo.append(u"\\psgrid[subgriddiv=2, gridlabels=0pt]")
    exo.append(u"\\psset{unit=10mm,arrowscale=2}")

    cor.append(u"\\begin{pspicture*}(0,0)(18,10)")
    cor.append(u"\\psgrid[subgriddiv=2, gridlabels=0pt]")
    cor.append(u"\\psset{unit=10mm,arrowscale=2}")

    exo.append(u"\\psdot(" + str(pointx) + "," + str(pointy) + ")")

    if pointx < 18 and pointy < 10:
      nompoint = str(pointx+0.5) + "," + str(pointy+0.5)
    else:
      nompoint = str(pointx-0.5) + "," + str(pointy-0.5)

    exo.append(u"\\rput(" + nompoint + "){\\psframebox[linecolor=white, fillcolor=white, fillstyle=solid]{$A$}}")

    cor.append(u"\\psdot(" + str(pointx) + "," + str(pointy) + ")")
    cor.append(u"\\rput(" + nompoint + "){\\psframebox[linecolor=white, fillcolor=white, fillstyle=solid]{$A$}}")

    cor.append(u"\psline[linecolor=DarkBlue]{|->}(" + str(pointx) + "," + str(pointy) + ")(" + str(pointx + t[1].x) + ", " + str(pointy + t[1].y) + ")")

    bx = pointx + t[1].x
    by = pointy + t[1].y

    if bx < 18 and by < 10:
      nompoint = str(bx+0.5) + "," + str(by+0.5)
    else:
      nompoint = str(bx-0.5) + "," + str(by-0.5)

    cor.append(u"\\psdot(" + str(pointx + t[1].x) + "," + str(pointy + t[1].y) + ")")
    cor.append(u"\\rput(" + nompoint + "){\\psframebox[linecolor=white, fillcolor=white, fillstyle=solid]{$B$}}")

    for vec in [(u, posux, posuy, "u"), (v, posvx, posvy, "v"), (w, poswx, poswy, "w")]:
      exo.append(u"\\rput(" + str(vec[1]) + "," + str(vec[2]) + "){")
      exo.append(u"\psline{|->}(0, 0)(" + str(vec[0].x) + ", " + str(vec[0].y) + ")")

      exo.append(u"\\rput(" + AffNom(vec[0]) + ") \
                   {\\psframebox[linecolor=white, fillcolor=white, fillstyle=solid]{$\\overrightarrow{" + vec[3] + "}$}}")
      exo.append(u"}")
    exo.append(u"\\end{pspicture*}")


    for vec in [(u, posux, posuy, "u"), (v, posvx, posvy, "v"), (w, poswx, poswy, "w")]:
      if vec[0].y>0:
        plus = 1
      else:
        plus = 0
      cor.append(u"\\rput(" + str(vec[1]) + "," + str(vec[2]) + "){")
      cor.append(u"\\psline{|->}(0, 0)(" + str(vec[0].x) + ", " + str(vec[0].y) + ")")
      cor.append(u"\\psline[linestyle=dashed,linecolor=DarkRed](0, 0)(" + str(vec[0].x) + ", 0)(" + str(vec[0].x) + "," + str(vec[0].y) + ")")
      cor.append(u"\\rput(" + AffNom(vec[0]) + "){\\psframebox[linecolor=white, fillcolor=white, \
                   fillstyle=solid]{$\\overrightarrow{" + vec[3] + "}\\ (" + str(vec[0].x) + ";" + str(vec[0].y) + ")$}}")
      cor.append(u"}")
    cor.append(u"\\end{pspicture*}")

    exo.append("\\par")
    cor.append("\\par")
    exo.append(u"On se place dans un repère orthonormé et on considère les vecteurs $\\overrightarrow{u}$, $\\overrightarrow{v}$, et $\\overrightarrow{w}$ ci-dessous.")
    cor.append(u"On se place dans un repère orthonormé et on considère les vecteurs $\\overrightarrow{u}$, $\\overrightarrow{v}$, et $\\overrightarrow{w}$ ci-dessous.")

    exo.append("\\begin{enumerate}")
    cor.append("\\begin{enumerate}")

    exo.append(u"\\item Lire les coordonnées de chacun des vecteurs $\\overrightarrow{u}$, $\\overrightarrow{v}$, et $\\overrightarrow{w}$.")
    cor.append(u"\\item Lire les coordonnées de chacun des vecteurs $\\overrightarrow{u}$, $\\overrightarrow{v}$, et $\\overrightarrow{w}$.")

    cor.append("\\par")
    cor.append(u"Un petit rappel : l'abscisse d'un vecteur est la différence d'abscisse entre le fin et le début du vecteur. \
                 Concernant le vecteur $\\overrightarrow{u}$, son abscisse est $" + str(u.x) + u"$. \
                 On lit également son ordonnée : $" + str(u.x) + u"$. \
                 Donc les coordonnées de $\\overrightarrow{u}$ sont $(" + str(u.x) + ", " + str(u.y) + u" )$. \
                 Des pointillés ont été ajoutés sur la figure pour faciliter la lecture des coordonnées.")
    cor.append(u"De même, les coordonnées de $\\overrightarrow{v}$ sont $(" + str(v.x) + ", " + str(v.y) + u" )$ \
                 et les coordonnées de $\\overrightarrow{w}$ sont $(" + str(w.x) + ", " + str(w.y) + " )$.")

    exo.append(u"\\item Placer un point B de sorte que le vecteur $\\overrightarrow{AB}$ soit égal à $" + str(t[0]) + " \\times \\overrightarrow{" + t[2] + "}$.")
    cor.append(u"\\item Placer un point B de sorte que le vecteur $\\overrightarrow{AB}$ soit égal à $" + str(t[0]) + " \\times \\overrightarrow{" + t[2] + "}$.")

    cor.append(u"\\par")
    cor.append(u"Le plus simple pour répondre à cette question est de calculer les coordonnées du vecteur $" + str(t[0]) + " \\times \\overrightarrow{" + str(t[2]) + "}$.")
    cor.append(u"Cela se fait en multipliant les coordonnées de $\\overrightarrow{" + str(t[2]) + "}$ par $" + str(t[0]) + u"$, ce qui donne comme résultat $(" + str(t[1].x) + ";" + str(t[1].y) + ")$.")
    cor.append(u"En partant du point A et en respectant ces coordonnées, on dessine un vecteur (en bleu sur la figure ci-dessus) qui indique l'emplacement du point B.")

    exo.append(u"\\item Calculer les normes de chacun des vecteurs $\\overrightarrow{u}$, $\\overrightarrow{v}$, et $\\overrightarrow{w}$.")
    cor.append(u"\\item Calculer les normes de chacun des vecteurs $\\overrightarrow{u}$, $\\overrightarrow{v}$, et $\\overrightarrow{w}$.")

    if u.x**2+u.y**2 == simplifie_racine(u.x**2+u.y**2)[1]: # Cas où la simplification est la même, donc inutile d'écrire deux fois la même chose.
      Norm_u = "$"
    else:
      Norm_u = "=" + str(u.normeTex()) + "$"

    if v.x**2+v.y**2 == simplifie_racine(v.x**2+v.y**2)[1]:
      Norm_v = "$"
    else:
      Norm_v = "=" + str(v.normeTex()) + "$"

    if w.x**2+w.y**2 == simplifie_racine(w.x**2+w.y**2)[1]:
      Norm_w = "$"
    else:
      Norm_w = "=" + str(w.normeTex()) + "$"

    cor.append("\\par")
    cor.append(u"$\|\\overrightarrow{u}\|=\\sqrt{(" + str(u.x) + ")^2+(" + str(u.y) + ")^2}=\\sqrt{" + str(u.x**2) + " + " + str(u.y**2) + "}= \
                 \\sqrt{" + str(u.x**2 + u.y**2) + "}" + Norm_u + ".\\par")
    cor.append(u"De la même manière, on obtient :")

    cor.append(u"$\|\\overrightarrow{v}\|=\\sqrt{(" + str(v.x) + ")^2+(" + str(v.y) + ")^2}=\\sqrt{" + str(v.x**2) + " + " + str(v.y**2) + "}= \
                 \\sqrt{" + str(v.x**2 + v.y**2) + "}" + Norm_v + " et \\par")
    cor.append(u"$\|\\overrightarrow{w}\|=\\sqrt{(" + str(w.x) + ")^2+(" + str(w.y) + ")^2}=\\sqrt{" + str(w.x**2) + " + " + str(w.y**2) + "}= \
                 \\sqrt{" + str(w.x**2 + w.y**2) + "}" + Norm_w + ".\\par")

    exo.append(u"\\item Dessiner des représentants des vecteurs $\\overrightarrow{u}+\\overrightarrow{v}$, $\\overrightarrow{u}-\\overrightarrow{v}$, $\\overrightarrow{u}-\\overrightarrow{w}$ \
                 et $\\overrightarrow{v}+\\overrightarrow{w}$.")
    cor.append(u"\\item Dessiner des représentants des vecteurs $\\overrightarrow{u}+\\overrightarrow{v}$, $\\overrightarrow{u}-\\overrightarrow{v}$, $\\overrightarrow{u}-\\overrightarrow{w}$ \
                 et $\\overrightarrow{v}+\\overrightarrow{w}$.")

    cor.append("\\par")
    cor.append(u"Pour dessiner les sommes ou différences de vecteurs, il faut les mettre \"bouts à bouts\", \
                 comme sur les figures qui suivent :\\par")
    i = repr_somme(u,v,'u','v',cor)
    repr_somme(u,-v,'u','-v',cor,i)
    cor.append("\\par")
    i = repr_somme(u,-w,'u','-w',cor)
    repr_somme(v,w,'v','w',cor,i)

    exo.append("\\end{enumerate}")
    cor.append("\\end{enumerate}")

    return exo,cor
